/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include <cassert>

#include "StdInclude.h"
#include "EbmlElement.h"

START_LIBEBML_NAMESPACE

/*!
	\todo verify that the size written is the same as the data written
*/
uint32 EbmlElement::Render(IOCallback & output)
{
	try {
		uint32 result = RenderHead(output);
	uint64 SupposedSize = UpdateSize();
		uint64 WrittenSize = RenderData(output);
	if (SupposedSize != (0-1)) assert(WrittenSize == SupposedSize);
		result += WrittenSize;
		return result;
	} catch (...) {
		return 0;
	}
}

/*!
	\todo handle exceptions on errors
	\todo handle CodeSize bigger than 5 bytes
*/
uint32 EbmlElement::RenderHead(IOCallback & output)
{
	binary FinalHead[4+8]; // Class D + 64 bits coded size
	unsigned int FinalHeadSize;
	
	if (EbmlId(*this).Length <= 0 || EbmlId(*this).Length > 4)
		return 0;
	
	for (FinalHeadSize = 0; FinalHeadSize<EbmlId(*this).Length; FinalHeadSize++) {
		FinalHead[FinalHeadSize] = EbmlId(*this).Value[FinalHeadSize];
	}
	
	UpdateSize();
	
	int CodedSize = CodedSizeLength();
	// Set the EBML bits
	FinalHead[FinalHeadSize] = 1 << (8 - CodedSize);
	
	uint64 TempSize = GetSize();
	int SizeMask = 0xFF;
	for (int i=1; i<CodedSize; i++) {
		FinalHead[FinalHeadSize + CodedSize - i] = TempSize & 0xFF;
		TempSize >>= 8;
		SizeMask >>= 1;
	}
//	SizeMask <<= 1;

	// first one use a OR with the "EBML size head"
	FinalHead[FinalHeadSize] |= TempSize & 0xFF & SizeMask;

	FinalHeadSize += CodedSize;
	
	output.writeFully(FinalHead, FinalHeadSize);
	
	return FinalHeadSize;
}

uint64 EbmlElement::ElementSize() const
{
	return Size + EbmlId(*this).Length + CodedSizeLength();
}

END_LIBEBML_NAMESPACE
