/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_ELEMENT_H
#define LIBEBML_ELEMENT_H

#include <string>

#include "EbmlTypes.h"
#include "EbmlRawElement.h"

START_LIBEBML_NAMESPACE

/*!
    \class EbmlElement
    \brief Pure virtual class to handle all Semantic operations on an EbmlElement
*/
class EbmlElement : public EbmlRawElement {
	public:
		EbmlElement(const uint64 DefaultSize) : EbmlRawElement(DefaultSize) {}
		virtual ~EbmlElement() {}
	
		uint64 ElementSize() const; /// return the size of the header+data
		
		uint32 Render(IOCallback & output);

		virtual uint64 UpdateSize() = 0; /// update the Size of the Data stored
		virtual uint64 GetSize() const {return Size;}

		virtual uint64 ReadData(IOCallback & input) = 0;
		
		/// return the generic callback to monitor a derived class
		virtual const EbmlCallbacks & Generic() const = 0;


	protected:
		uint32 RenderHead(IOCallback & output);
	
		virtual uint32 RenderData(IOCallback & output) = 0;
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_ELEMENT_H
