/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_FLOAT_H
#define LIBEBML_FLOAT_H

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

START_LIBEBML_NAMESPACE

/*!
    \class EbmlFloat
    \brief Handle all operations on a float EBML element
*/
class EbmlFloat : public EbmlElement {
	public:
		enum Precision {
			 FLOAT_32
			,FLOAT_64
#if LIBEBML_SUPPORT_FLOAT80
			,FLOAT_80
#endif // LIBEBML_SUPPORT_FLOAT80
		};

		EbmlFloat(const Precision prec = FLOAT_32, const double DefaultValue = 0.0);
		EbmlFloat(const EbmlRawElement & ElementToClone, double DefaultValue = 0.0);
	
		uint32 RenderData(IOCallback & output);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize();

		void SetPrecision(const EbmlFloat::Precision prec = FLOAT_32) 
		{
			if (prec == FLOAT_64)
				Size = 8;
#if LIBEBML_SUPPORT_FLOAT80
			else if (prec == FLOAT_80)
				Size = 10;
#endif // LIBEBML_SUPPORT_FLOAT80
			else
				Size = 4; // default size
		}

	
//		EbmlFloat & operator=(const float NewValue) { Value = NewValue; return *this;}
		EbmlFloat & operator=(const double NewValue) { Value = NewValue; return *this;}
		
		operator const float() const {return float(Value);}
		operator const double() const {return double(Value);}
	
	protected:
		double Value; /// The actual value of the element
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_FLOAT_H
