/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_RAW_ELEMENT_H
#define LIBEBML_RAW_ELEMENT_H

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "IOCallback.h"
#include "EbmlId.h"

START_LIBEBML_NAMESPACE

class EbmlElement;

// functions for generic handling of data (should be static to all classes)
class EbmlCallbacks {
	public:
		EbmlCallbacks(EbmlElement & (*Creator)(), const EbmlId & aGlobalId, const std::string & aDebugName, const struct EbmlSemanticContext & aContext)
			:Create(Creator)
			,GlobalId(aGlobalId)
			,DebugName(aDebugName)
			,Context(aContext)
		{}

		EbmlElement & (*Create)();
		const EbmlId & GlobalId;
		const std::string & DebugName;
		const struct EbmlSemanticContext & Context;
};

/*!
	\brief contains the semantic informations for a given level and all sublevels
	\todo move the ID in the element class
*/
struct EbmlSemantic {
	bool Mandatory; ///< wether the element is mandatory in the context or not
	bool Unique;
	EbmlCallbacks * GetCallbacks;
};

/*!
	Context of the element
	\todo allow more than one parent ?
*/
struct EbmlSemanticContext {
	unsigned int Size;          ///< number of elements in the table
	const struct EbmlSemantic *MyTable; ///< First element in the table
	const struct EbmlSemanticContext *UpTable; ///< Parent element
};

/*!
    \class EbmlRawElement
    \brief Hold basic informations about an EBML element (ID + length)
		\todo Merge with EbmlElement ?
*/
class EbmlRawElement {
	public:
		EbmlRawElement(const uint64 DefaultSize);
	
		/// Set the minimum length that will be used to write the element size (-1 = optimal)
		void SetSizeLength(const int NewSizeLength) {SizeLength = NewSizeLength;}
		
		static EbmlRawElement * FindNextID(IOCallback & DataStream, const struct EbmlSemanticContext & Context, const uint64 MaxDataSize, const bool AllowUnknownSize = true);
		static EbmlRawElement * FindNextID(IOCallback & DataStream, const struct EbmlSemanticContext & Context, const EbmlCallbacks & ClassInfos, const uint64 MaxDataSizeconst, const bool AllowUnknownSize);
		/*!
			\brief find the next element with the same ID
		*/
		EbmlRawElement * FindNext(IOCallback & DataStream, const uint64 MaxDataSize, const bool AllowUnknownSize = true);

		void SkipData(IOCallback & DataStream);

		virtual operator const EbmlId &() const = 0;

	protected:
		uint64 Size;     /// the size of the data (0 = optimal)
		bool bSizeOptimal;
		int SizeLength; /// the minimum size on which the size will be written (0 = optimal)
		int CodedSizeLength() const;
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_RAW_ELEMENT_H
