/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include <cassert>

#include "StdInclude.h"
#include "EbmlSInteger.h"

START_LIBEBML_NAMESPACE

EbmlSInteger::EbmlSInteger(const int64 DefaultValue, const int DefaultSize)
 :EbmlElement(DefaultSize), Value(DefaultValue)
{}

EbmlSInteger::EbmlSInteger(const EbmlRawElement & ElementToClone, int64 DefaultValue)
 :EbmlElement(0), Value(DefaultValue)
{
	*(static_cast<EbmlRawElement*>(this)) = ElementToClone;
}

/*!
	\todo handle exception on errors
*/
uint32 EbmlSInteger::RenderData(IOCallback & output)
{
	binary FinalData[8]; // we don't handle more than 64 bits integers
	unsigned int i;
	
	if (SizeLength > 8)
		return 0; // integer bigger coded on more than 64 bits are not supported
	
	int64 TempValue = Value;
	for (i=0; i<Size;i++) {
		FinalData[Size-i-1] = TempValue & 0xFF;
		TempValue >>= 8;
	}
	
	output.writeFully(FinalData,Size);

	return Size;
}

uint64 EbmlSInteger::UpdateSize()
{
	if (bSizeOptimal)
		Size = CodedSizeLength();
	return Size;
}

uint64 EbmlSInteger::ReadData(IOCallback & input)
{
	binary Buffer[8];
	input.readFully(Buffer, Size);
	
	if (Buffer[0] & 0x80)
		Value = -1; // this is a negative value
	else
		Value = 0; // this is a positive value
	
	for (unsigned int i=0; i<Size; i++)
	{
		Value <<= 8;
		Value |= Buffer[i];
	}

	return Size;
}

END_LIBEBML_NAMESPACE
