/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_SINTEGER_H
#define LIBEBML_SINTEGER_H

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

START_LIBEBML_NAMESPACE

const int DEFAULT_INT_SIZE = 1; ///< optimal

/*!
    \class EbmlSInteger
    \brief Handle all operations on a signed integer EBML element
*/
class EbmlSInteger : public EbmlElement {
	public:
		EbmlSInteger(int64 DefaultValue = 0, const int DefaultSize = DEFAULT_INT_SIZE);
		EbmlSInteger(const EbmlRawElement & ElementToClone, int64 DefaultValue = 0);
	
		EbmlSInteger & operator=(const uint64 NewValue) {Value = NewValue; return *this;}

		/*!
			Set the default size of the integer (usually 1,2,4 or 8)
		*/
		void SetDefaultSize(const int DefaultSize = DEFAULT_INT_SIZE) {Size = DefaultSize;}

		uint32 RenderData(IOCallback & output);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize();
	
		operator int8() {return Value;}
		operator int16() {return Value;}
		operator int32() {return Value;}
		operator int64() {return Value;}

	protected:
		int64 Value; /// The actual value of the element
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_SINTEGER_H
