/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include <cassert>

#include "StdInclude.h"
#include "EbmlString.h"

START_LIBEBML_NAMESPACE

EbmlString::EbmlString(const std::string & DefaultValue, const uint32 aDefaultSize)
 :EbmlElement(aDefaultSize), Value(DefaultValue)
{
	DefaultSize = aDefaultSize;
/* done automatically	
	Size = Value.length();
	if (DefaultSize > Size)
		Size = DefaultSize;*/
}

/*!
	\todo Cloning should be on the same exact type !
*/
EbmlString::EbmlString(const EbmlRawElement & ElementToClone, const std::string & aDefaultValue)
 :EbmlElement(0), Value(aDefaultValue)
{
	*(static_cast<EbmlRawElement*>(this)) = ElementToClone;
}

/*!
	\todo handle exception on errors
*/
uint32 EbmlString::RenderData(IOCallback & output)
{
	uint32 Result = 0;
	output.writeFully(Value.c_str(), Value.length());
	Result += Value.length();
	
	if (Value.length() < DefaultSize) {
		// pad the rest with 0
		binary *Pad = new binary[DefaultSize - Value.length()];
		if (Pad == NULL)
		{
			return Result;
		}
		memset(Pad, 0x00, DefaultSize - Value.length());
		output.writeFully(Pad, DefaultSize - Value.length());
		delete Pad;
		Result = DefaultSize;
	}
	
	return Result;
}

EbmlString & EbmlString::operator=(const std::string NewString)
{
	Value = NewString;
/* done automatically	
	Size = Value.length();
	if (DefaultSize > Size)
		Size = DefaultSize;*/
	return *this;
}

uint64 EbmlString::UpdateSize()
{
	if (Value.length() < DefaultSize) {
		Size = DefaultSize;
	} else {
		Size = Value.length();
	}
	return Size;
}

uint64 EbmlString::ReadData(IOCallback & input)
{
	char *Buffer = new char[Size + 1];
	if (Buffer == NULL) {
		// unable to store the data, skip it
		input.setFilePointer(Size, seek_current);
	} else {
		input.readFully(Buffer, Size);
		if (Buffer[Size-1] != '\0') {
			Buffer[Size] = '\0';
		}
		Value = Buffer;
		delete Buffer;
	}

	return Size;
}

END_LIBEBML_NAMESPACE
