/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_STRING_H
#define LIBEBML_STRING_H

#include <string>

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

START_LIBEBML_NAMESPACE

/*!
    \class EbmlString
    \brief Handle all operations on a printable string EBML element
*/
class EbmlString : public EbmlElement {
	public:
		EbmlString(const std::string & aDefaultValue = "", const uint32 aDefaultSize = 0);
		EbmlString(const EbmlRawElement & ElementToClone, const std::string & aDefaultValue = "");
	
		virtual ~EbmlString() {}
	
		uint32 RenderData(IOCallback & output);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize();
	
		EbmlString & operator=(const std::string);
		operator const std::string &() const {return Value;}
	
	protected:
		std::string Value;  /// The actual value of the element
		uint32 DefaultSize; /// Minimum data size to fill on rendering
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_STRING_H
