/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.h"
#include "EbmlSubHead.h"
#include "EbmlContexts.h"

START_LIBEBML_NAMESPACE

EbmlId EVersion::TheId            = {{0x42, 0x86}, 2 };
EbmlId EReadVersion::TheId        = {{0x42, 0xF7}, 2 };
EbmlId EMaxIdLength::TheId        = {{0x42, 0xF2}, 2 };
EbmlId EMaxSizeLength::TheId      = {{0x42, 0xF3}, 2 };
EbmlId EDocType::TheId            = {{0x42, 0x82}, 2 };
EbmlId EDocTypeVersion::TheId     = {{0x42, 0x87}, 2 };
EbmlId EDocTypeReadVersion::TheId = {{0x42, 0x85}, 2 };

EbmlCallbacks EVersion::ClassInfos(EVersion::Create,               EVersion::TheId,        "EBMLVersion",                        EVersion_Context);
EbmlCallbacks EReadVersion::ClassInfos(EReadVersion::Create,       EReadVersion::TheId,    "EBMLReadVersion",                    EReadVersion_Context);
EbmlCallbacks EMaxIdLength::ClassInfos(EMaxIdLength::Create,       EMaxIdLength::TheId,    "EBMLMaxIdLength",                    EMaxIdLength_Context);
EbmlCallbacks EMaxSizeLength::ClassInfos(EMaxSizeLength::Create,   EMaxSizeLength::TheId,  "EBMLMaxSizeLength",                  EMaxSizeLength_Context);
EbmlCallbacks EDocType::ClassInfos(EDocType::Create,               EDocType::TheId,        "EBMLDocType",                        EDocType_Context);
EbmlCallbacks EDocTypeVersion::ClassInfos(EDocTypeVersion::Create, EDocTypeVersion::TheId, "EBMLDocTypeVersion",                 EDocTypeVersion_Context);
EbmlCallbacks EDocTypeReadVersion::ClassInfos(EDocTypeReadVersion::Create, EDocTypeReadVersion::TheId, "EBMLDocTypeReadVersion", EDocTypeReadVersion_Context);

struct EbmlSemanticContext EVersion_Context        = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EReadVersion_Context    = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EMaxIdLength_Context    = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EMaxSizeLength_Context  = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EDocType_Context        = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EDocTypeVersion_Context = {0, NULL, &EbmlHead_Context};
struct EbmlSemanticContext EDocTypeReadVersion_Context = {0, NULL, &EbmlHead_Context};

END_LIBEBML_NAMESPACE
