/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_SUBHEAD_H
#define LIBEBML_SUBHEAD_H

#include <string>

#include "EbmlUInteger.h"
#include "EbmlString.h"

START_LIBEBML_NAMESPACE

class EVersion : public EbmlUInteger {
	public:
		EVersion() :EbmlUInteger(1) {}
		static EbmlElement & Create() {return *(new EVersion);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EReadVersion : public EbmlUInteger {
	public:
		EReadVersion() :EbmlUInteger(1) {}
		static EbmlElement & Create() {return *(new EReadVersion);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EMaxIdLength : public EbmlUInteger {
	public:
		EMaxIdLength() :EbmlUInteger(4) {}
		static EbmlElement & Create() {return *(new EMaxIdLength);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EMaxSizeLength : public EbmlUInteger {
	public:
		EMaxSizeLength() :EbmlUInteger(8) {}
		static EbmlElement & Create() {return *(new EMaxSizeLength);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EDocType : public EbmlString {
	public:
		EDocType() {}
		static EbmlElement & Create() {return *(new EDocType);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EDocTypeVersion : public EbmlUInteger {
	public:
		EDocTypeVersion() {}
		static EbmlElement & Create() {return *(new EDocTypeVersion);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

class EDocTypeReadVersion : public EbmlUInteger {
	public:
		EDocTypeReadVersion() {}
		static EbmlElement & Create() {return *(new EDocTypeReadVersion);}
		const EbmlCallbacks & Generic() const {return ClassInfos;}
		static EbmlCallbacks ClassInfos;
		operator const EbmlId &() const {return ClassInfos.GlobalId;}
	
	protected:
		static EbmlId TheId;
		static std::string TheName;
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_SUBHEAD_H
