/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_UINTEGER_H
#define LIBEBML_UINTEGER_H

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

START_LIBEBML_NAMESPACE

const int DEFAULT_UINT_SIZE = 0; ///< optimal

/*!
    \class EbmlUInteger
    \brief Handle all operations on an unsigned integer EBML element
*/
class EbmlUInteger : public EbmlElement {
	public:
		EbmlUInteger(uint64 DefaultValue = 0, const int DefaultSize = DEFAULT_UINT_SIZE);
		EbmlUInteger(const EbmlRawElement & ElementToClone, uint64 DefaultValue = 0);
	
		EbmlUInteger & operator=(const uint64 NewValue) {Value = NewValue; return *this;}

		/*!
			Set the default size of the integer (usually 1,2,4 or 8)
		*/
		void SetDefaultSize(const int DefaultSize = DEFAULT_UINT_SIZE) {Size = DefaultSize;}

		uint32 RenderData(IOCallback & output);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize();
		
		operator uint8() {return Value;}
		operator uint16() {return Value;}
		operator uint32() {return Value;}
		operator uint64() {return Value;}

	protected:
		uint64 Value; /// The actual value of the element
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_UINTEGER_H
