/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include <clocale>
#include <cassert>

#include "StdInclude.h"
#include "EbmlUnicodeString.h"

START_LIBEBML_NAMESPACE

EbmlUnicodeString::EbmlUnicodeString(const UTFstring & DefaultValue, const uint32 aDefaultSize)
 :EbmlElement(aDefaultSize), Value(DefaultValue)
{
	DefaultSize = aDefaultSize;
	setlocale(LC_CTYPE, "UTF-8"); // done but not ensured :(
}

EbmlUnicodeString::EbmlUnicodeString(const EbmlRawElement & ElementToClone, const UTFstring & DefaultValue)
 :EbmlElement(0), Value(DefaultValue)
{
	*(reinterpret_cast<EbmlRawElement*>(this)) = ElementToClone;
}

/*!
	\todo handle conversion
	\todo handle exception on errors
*/
uint32 EbmlUnicodeString::RenderData(IOCallback & output)
{
	uint32 Result = 0;
	
	char * tmpStr = new char[2*Value.length()];
	std::string UTF8String;
	
	if (tmpStr != NULL) {
		::wcstombs(tmpStr, &Value[0], 2*Value.length());

		UTF8String = tmpStr;

		output.writeFully(UTF8String.c_str(), UTF8String.length());
		Result = UTF8String.length();

		if (UTF8String.length() < DefaultSize) {
			// pad the rest with 0
			binary *Pad = new binary[DefaultSize - UTF8String.length()];
			if (Pad != NULL) {
				memset(Pad, 0x00, DefaultSize - UTF8String.length());
				output.writeFully(Pad, DefaultSize - UTF8String.length());
				delete Pad;

				Result = DefaultSize;
			}
		}

		delete tmpStr;
	}

	return Result;
}

EbmlUnicodeString & EbmlUnicodeString::operator=(const UTFstring NewString)
{
	Value = NewString;
	return *this;
}

/*!
	\todo Decode UTF-8 encoded to Wide char string (internal)
*/
EbmlUnicodeString & EbmlUnicodeString::operator=(const std::string NewString)
{
	wchar_t *tmpStr = new wchar_t[2*NewString.length()];
	if (tmpStr != NULL) {
		::mbstowcs(tmpStr, NewString.c_str(), 2*NewString.length());
		Value = tmpStr;
		delete tmpStr;
	}

	return *this;
}

uint64 EbmlUnicodeString::UpdateSize()
{
	char * tmpStr = new char[2*Value.length()];
	std::string UTF8String;
	Size = 0;
	
	if (tmpStr != NULL) {
		::wcstombs(tmpStr, &Value[0], 2*Value.length());
		UTF8String = tmpStr;
		Size = UTF8String.length();
		if (Size < DefaultSize) {
			Size = DefaultSize;
		}
	}
	
	return Size;
}

uint64 EbmlUnicodeString::ReadData(IOCallback & input)
{
	char *Buffer = new char[Size+1];
	if (Buffer == NULL) {
		// impossible to read, skip it
		input.setFilePointer(Size, seek_current);
	} else {
		input.readFully(Buffer, Size);
		if (Buffer[Size-1] != '\0') {
			Buffer[Size] = '\0';
		}
		
		wchar_t *tmpStr = new wchar_t[2*Size];
		::mbstowcs(tmpStr, Buffer, 2*Size);
		Value = tmpStr;
		delete tmpStr;
		delete Buffer;
	}

	return Size;
}

END_LIBEBML_NAMESPACE
