/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBEBML_UNICODE_STRING_H
#define LIBEBML_UNICODE_STRING_H

#include <string>

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

/*
#if defined(__GNUC__) // tested on cygwin
typedef basic_string<wchar_t> UTFstring;
#else
*/
typedef std::wstring UTFstring;
//#endif

START_LIBEBML_NAMESPACE

/*!
    \class EbmlUnicodeString
    \brief Handle all operations on a Unicode string EBML element
		\note internally treated as a string made of wide characters (16 bits ?)
*/
class EbmlUnicodeString : public EbmlElement {
	public:
		EbmlUnicodeString(const UTFstring & DefaultValue = L"", const uint32 aDefaultSize = 0);
		EbmlUnicodeString(const EbmlRawElement & ElementToClone, const UTFstring & aDefaultValue = L"");
	
		virtual ~EbmlUnicodeString() {}
	
		uint32 RenderData(IOCallback & output);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize();
	
		EbmlUnicodeString & operator=(const UTFstring); ///< platform dependant code
		EbmlUnicodeString & operator=(const std::string); ///< UTF-8 to internal/platform
// should not exist		operator const std::string &() const;
		operator const UTFstring &() const {return Value;}
	
	protected:
		UTFstring Value; /// The actual value of the element
		uint32 DefaultSize; /// Minimum data size to fill on rendering
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_UNICODE_STRING_H
