/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.h"
#include "KaxAttached.h"
#include "KaxContexts.h"

// sub elements

using namespace LIBEBML_NAMESPACE;

START_LIBMATROSKA_NAMESPACE

struct EbmlSemantic KaxAttached_ContextList[4] =
{
	{true,  true, &KaxFileName::ClassInfos},
	{true,  true, &KaxMimeType::ClassInfos},
	{true,  true, &KaxFileData::ClassInfos},
	{false, true, &KaxFileDescription::ClassInfos},
};

EbmlId KaxAttached::TheId = {{0x61, 0xA7}, 2 };
EbmlId KaxFileDescription::TheId = {{0x46, 0x7E}, 2 };
EbmlId KaxFileName::TheId = {{0x46, 0x6E}, 2 };
EbmlId KaxMimeType::TheId = {{0x46, 0x60}, 2 };
EbmlId KaxFileData::TheId = {{0x46, 0x5C}, 2 };

EbmlCallbacks KaxAttached::ClassInfos(KaxAttached::Create, KaxAttached::TheId, "AttachedFile", KaxAttached_Context);
EbmlCallbacks KaxFileDescription::ClassInfos(KaxFileDescription::Create, KaxFileDescription::TheId, "FileDescription", KaxFileDescription_Context);
EbmlCallbacks KaxFileName::ClassInfos(KaxFileName::Create, KaxFileName::TheId, "FileName", KaxFileName_Context);
EbmlCallbacks KaxMimeType::ClassInfos(KaxMimeType::Create, KaxMimeType::TheId, "FileMimeType", KaxMimeType_Context);
EbmlCallbacks KaxFileData::ClassInfos(KaxFileData::Create, KaxFileData::TheId, "FileData", KaxFileData_Context);

struct EbmlSemanticContext KaxAttached_Context = {countof(KaxAttached_ContextList), KaxAttached_ContextList, &KaxAttachements_Context};
struct EbmlSemanticContext KaxFileDescription_Context = {0, NULL, &KaxAttachements_Context};
struct EbmlSemanticContext KaxFileName_Context        = {0, NULL, &KaxAttachements_Context};
struct EbmlSemanticContext KaxMimeType_Context        = {0, NULL, &KaxAttachements_Context};
struct EbmlSemanticContext KaxFileData_Context        = {0, NULL, &KaxAttachements_Context};

KaxAttached::KaxAttached()
	:EbmlMaster()
{
	SetSizeLength(2); // mandatory min size support (for easier updating) (2^(7*2)-2 = 16Ko)
	Context = &KaxAttached_Context;
	ProcessMandatory();
}

END_LIBMATROSKA_NAMESPACE
