/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Ingo Ralf Blum

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: Config.hpp,v 1.12 2002/05/17 18:02:18 robux4 Exp $
*/
#ifndef LIBMATROSKA_CONFIG_H
#define LIBMATROSKA_CONFIG_H

// We allow the user to set the namespace for the library for cases, where the name conflicts
#ifdef NO_NAMESPACE // for older GCC

#define LIBMATROSKA_NAMESPACE libmatroska
#define START_LIBMATROSKA_NAMESPACE
#define END_LIBMATROSKA_NAMESPACE
#else // NO_NAMESPACE

#ifndef LIBMATROSKA_NAMESPACE
#define LIBMATROSKA_NAMESPACE libmatroska
#define START_LIBMATROSKA_NAMESPACE namespace LIBMATROSKA_NAMESPACE {
#define END_LIBMATROSKA_NAMESPACE   };
#endif // LIBMATROSKA_NAMESPACE

#endif // NO_NAMESPACE

// There are special implementations for certain platforms. For example on Windows
// we use the Win32 file API. here we set the appropriate macros.
#if defined(_WIN32)||defined(WIN32)
#define LIBMATROSKA_WIN32
#else
#define LIBMATROSKA_UNIX
#endif



// We assume, that the user uses the DLL by default. If he wants to use the static library, the MATROSKA_STATIC value
// has to be specified to the preprocessor
#ifdef LIBMATROSKA_WIN32
	// Either MATROSKA_STATIC or MATROSKA_EXPORT should be defined here.
	#ifdef LIBMATROSKA_STATIC
		#define LIBMATROSKA_EXPORT
	#else
		#ifndef LIBMATROSKA_EXPORT
//			#define LIBMATROSKA_EXPORT __declspec(dllimport) // This should work on Visual C++ and GCC (MinGW and Cygwin)
			#define LIBMATROSKA_EXPORT __declspec(dllexport) // This should work on Visual C++ and GCC (MinGW and Cygwin)
		#endif
	#endif // LIBMATROSKA_STATIC
#else
	// On other platforms we usually don't need the dllimport and dllexport directives
	#define LIBMATROSKA_EXPORT
#endif // LIBMATROSKA_WIN32


// The MATROSKA_DEBUG symbol is defined, when we are creating a debug build. In this
// case the debug logging code is compiled in.
#if (defined(DEBUG)||defined(_DEBUG))&&!defined(LIBMATROSKA_DEBUG)
#define LIBMATROSKA_DEBUG
#endif

#endif // LIBMATROSKA_CONFIG_H
