/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id$
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.h"
#include "KaxSegment.h"

// sub elements
#include "KaxCluster.h"
#include "KaxSeekHead.h"
#include "KaxCues.h"
#include "KaxTracks.h"
#include "KaxInfo.h"
#include "KaxChapters.h"
#include "KaxAttachements.h"
#include "KaxContexts.h"

START_LIBMATROSKA_NAMESPACE

struct EbmlSemantic KaxSegment_ContextList[] =
{
	{false, false, &KaxCluster::ClassInfos},
	{false, false, &KaxSeekHead::ClassInfos},
	{false, true,  &KaxCues::ClassInfos},
	{false, false, &KaxTracks::ClassInfos},
	{false, false, &KaxInfo::ClassInfos},
	{false, true,  &KaxChapters::ClassInfos},
	{false, true,  &KaxAttachements::ClassInfos},
};

struct EbmlSemanticContext KaxSegment_Context = {countof(KaxSegment_ContextList), KaxSegment_ContextList, NULL};

EbmlId KaxSegment::TheId = {{0x18, 0x53, 0x80, 0x67}, 4 };
EbmlCallbacks KaxSegment::ClassInfos(KaxSegment::Create, KaxSegment::TheId, "Segment", KaxSegment_Context);

KaxSegment::KaxSegment()
	:EbmlMaster()
{
	SetSizeLength(5); // mandatory min size support (for easier updating) (2^(7*5)-2 = 32Go)
	SetSizeInfinite(); // by default a segment is big and the size is unknown in advance
	Context = &KaxSegment_Context;
	
	ProcessMandatory();
}

END_LIBMATROSKA_NAMESPACE
