/*
	libmatroska, a library to parse/write Matroska files, see http://matroska.sourceforge.net/
	Copyright (c) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*!
    \file libmatroska.c
    \version \$Id: libmatroska.cpp,v 1.2 2002/05/17 18:02:18 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>

    \brief C wrapper to libmatroska
*/

#include "StdInclude.hpp"

#include "libmatroska.h"

#include "StdIOCallback.hpp"
#include "FileKax.hpp"
#include "Track.hpp"
#include "TrackType.hpp"
#include "TrackAudio.hpp"
//#include "TrackAV.hpp"
//#include "TrackControl.hpp"
//#include "TrackCustom.hpp"
//#include "TrackMultiAudio.hpp"
#include "TrackSubtitle.hpp"
//#include "TrackTitle.hpp"
//#include "TrackVideo.hpp"

using namespace LIBMATROSKA_NAMESPACE;

matroska_stream matroska_open_stream_file(c_string string, open_mode mode)
{
    // write the head of the file (with everything already configured)
    StdIOCallback *fileid = new StdIOCallback(string,mode);
    return fileid;
}

matroska_id matroska_open_stream(matroska_stream a_stream)
{
    IOCallback *_stream = (IOCallback *)a_stream;
    // create the Matroska file
    FileKax *aFile = new FileKax(*_stream);

    return aFile;
}

void matroska_close(matroska_id id)
{
    if (id != NULL)
    {
	FileKax * File = (FileKax *) id;
	delete File;
    }
}

void matroska_end(matroska_id id, uint32 totaltime)
{
    if (id != NULL)
    {
	FileKax * File = (FileKax *) id;
	File->Close(totaltime);
	delete File;
    }
}

matroska_track matroska_create_track(matroska_id id, const track_type aType)
{
    FileKax * File = (FileKax *) id;
    Track * binID = File->CreateTrack( aType );

    return binID;
}

void matroska_read_head(matroska_id id)
{
    FileKax * File = (FileKax *) id;
    File->ReadHead( );
}

void matroska_read_tracks(matroska_id id)
{
    FileKax * File = (FileKax *) id;
    File->ReadTracks( );
}

uint8 matroska_get_number_track(matroska_id id)
{
    FileKax * File = (FileKax *) id;
    return File->GetTrackNumber();
}

matroska_track matroska_get_track(matroska_id id, uint8 track_index)
{
    FileKax * File = (FileKax *) id;
    return File->GetTrack(track_index);
}

void matroska_get_track_info(matroska_id id, matroska_track track, track_info * infos)
{
    FileKax * File = (FileKax *) id;
    Track * _Track = (Track *) track;

    TrackInfo _infos;
    File->Track_GetInfo(_Track, _infos);

    infos->Enabled = _infos.Enabled;
    infos->Preferred = _infos.Preferred;
    infos->TrackType = track_type(_infos.TrackType);
    infos->Version1 = _infos.Version1;
    infos->Version2 = _infos.Version2;
    infos->Version3 = _infos.Version3;
    memcpy(infos->LanguageCode, _infos.LanguageCode, 4);
    memcpy(infos->CodecName, _infos.CodecName.c_str(), min(sizeof(infos->CodecName), _infos.CodecName.length()+1));
    memcpy(infos->CodecUrl, _infos.CodecUrl.c_str(), min(sizeof(infos->CodecUrl), _infos.CodecUrl.length()+1));
    memcpy(infos->CodecUrl_alt, _infos.CodecUrl_alt.c_str(), min(sizeof(infos->CodecUrl_alt), _infos.CodecUrl_alt.length()+1));
    memcpy(infos->Format, _infos.Format.c_str(), min(sizeof(infos->Format), _infos.Format.length())+1);
    memcpy(infos->CodecName, _infos.CodecName.c_str(), min(sizeof(infos->CodecName), _infos.CodecName.length()+1));
    memcpy(infos->CodecSettings, _infos.CodecSettings.c_str(), min(sizeof(infos->CodecSettings), _infos.CodecSettings.length()+1));
    memcpy(infos->Name, _infos.Name.c_str(), min(sizeof(infos->Name), _infos.Name.length()+1));
}
