/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Ingo Ralf Blum

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: ActualHeader.hpp,v 1.16 2002/07/14 20:41:40 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_ACTUAL_HEADER_HPP
#define LIBMATROSKA_ACTUAL_HEADER_HPP

#include <vector>

#include "Config.hpp"
#include "Types.hpp"
#include "TrackEntry.hpp"
#include "IOCallback.hpp"
#include "FileName.hpp"
#include "StreamInfo.hpp"

START_MATROSKA_NAMESPACE

typedef binary SmallElement[4];

/*!
    \class ActualHeader
    \brief The main documentation header of the Matroska file
    \see http://matroska.sourceforge.net/specs/#actual
*/
class ActualHeader {
    public:
	enum file_use {
	    USE_ALL,
	    USE_READ,
	    USE_NONE,
	};

	/*!
	    \todo change the version number accordingly to the latest spec
	 */
	ActualHeader()
	    :myMinVersion(myVersion)
	    ,myAbsMinVersion(myVersion)
	    ,myLinearWriteMode(false)
	    ,myOriginalFilename(64)
	    ,myNextPartFilename(64)
	    ,myNextPartTimecode(0)
	    ,myUseMode(USE_ALL)
	{}

	~ActualHeader() {}

	uint8 CreateTrack();

	static inline uint32 default_size() {return 864;}

	uint32 Render(IOCallback & output, const StreamInfo & aStreamInfo) const;

	/*!
	    \return the number of bytes read from file
	*/
	uint32 Read(IOCallback & input, StreamInfo & aStreamInfo);

    protected:
	static const uint8 myVersion;
	uint8 myMinVersion;
	uint8 myAbsMinVersion;
	bool myLinearWriteMode;

	static const SmallElement myTrackElement;
	static const SmallElement myCodecElement;
	static const SmallElement mySeekElement;
	static const SmallElement myChapterElement;
	static const SmallElement myAttachElement;
	static const SmallElement mySignatureElement;

	FileName myOriginalFilename;

	FileName myNextPartFilename;
	uint32   myNextPartTimecode;

	file_use myUseMode;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_ACTUAL_HEADER_HPP
