/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: Adler32.cpp,v 1.3 2002/05/01 08:13:22 robux4 Exp $
*/
#include "StdInclude.hpp"
//#include "Config.hpp"
#include "Adler32.hpp"

START_MATROSKA_NAMESPACE

const checksum Adler32::BASE = 65521L;
const size_t   Adler32::NMAX = 5552;

// some macro that comes from the zlib code
#define DO1(buf,i)  {s1 += buf[i]; s2 += s1;}
#define DO2(buf,i)  DO1(buf,i); DO1(buf,i+1);
#define DO4(buf,i)  DO2(buf,i); DO2(buf,i+2);
#define DO8(buf,i)  DO4(buf,i); DO4(buf,i+4);
#define DO16(buf)   DO8(buf,0); DO8(buf,8);

/*!
	\brief compute the new Adler32 checksum with the additional data
	\note The code is based on the zlib code
*/
void Adler32::Feed(const void * buffer, const size_t bufferSize)
{
    if (buffer == NULL) return;

    uint32 val = value;

    int k;
    unsigned long s1 = val & 0xffff;
    unsigned long s2 = (val >> 16) & 0xffff;
    size_t len = bufferSize;
    binary * buf = (binary *)buffer;

    while (len > 0) {
        k = len < NMAX ? len : NMAX;
	len -= k;
	while (k >= 16) {
	    DO16(buf);
	    buf += 16;
	    k -= 16;
	}
	if (k != 0) do {
	    s1 += *buf++;
	    s2 += s1;
	} while (--k);
	s1 %= BASE;
	s2 %= BASE;
    }
    val = (s2 << 16) | s1;

    value=val;
}

END_MATROSKA_NAMESPACE
