/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Ingo Ralf Blum

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef LIBMATROSKA_ADLER32_HPP
#define LIBMATROSKA_ADLER32_HPP

#include "Config.hpp"
#include "Types.hpp"

START_MATROSKA_NAMESPACE

/*!
    \class Adler32
    \brief Compute Adler32 checksum
    \version \$Id: Adler32.hpp,v 1.8 2002/05/12 16:04:52 robux4 Exp $
    \note the Adler-32 checksum is saved as a big endian
    \see http://www.cis.ohio-state.edu/cgi-bin/rfc/rfc1950.html - bottom of the page: info and C-code for calculating
    \see http://www.gzip.org/zlib/zlib_tech.html - bottom of that page: CRC-32 against Adler-32
*/
class Adler32 {
    public:
	Adler32(const checksum initial=1):value(initial) {}

	void Feed(const void * buffer, const size_t bufferSize);
	inline uint32 & endian() { return value.endian(); }
	inline uint32 size() {return 4;}

	bool operator!=(const binary *buffer) const { 
	    checksum aValue;
	    aValue.Eval(buffer);
	    return value != aValue; 
	}

    protected:
	checksum value;

	/// largest prime smaller than 65536
	static const checksum BASE;
	static const size_t NMAX;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_ADLER32_HPP
