/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: Block.cpp,v 1.15 2002/07/14 20:44:23 robux4 Exp $
    \author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "Block.hpp"

START_MATROSKA_NAMESPACE

Block::Block(const uint8 track_number, const uint32 timecode)
 :TrackNumber(track_number)
 ,TimeCode(timecode)
{}

Block::Block()
 :TrackNumber(0)
 ,TimeCode(0)
{}

Block::Block(const uint8 track_number, const binary * aFrame, const uint32 aFrameSize)
{
    // block size
    big_uint32 block_size;
    block_size.Eval(&aFrame[0x00]);
    mySize = block_size;

    if (mySize != aFrameSize)
	throw 0; // there must be a problem somewhere

    // track number
    TrackNumber = aFrame[0x08];

    if (TrackNumber != track_number)
	throw 0; // there must be a problem somewhere

    // timecode
    big_uint32 time_code;
    time_code.Eval(&aFrame[0x04]);
    TimeCode = time_code;


    // flags
    uint8 flags = aFrame[0x09];
    KeyFrame = (flags & 0x40) == 0;
    BFrame = (flags & 0x20) == 0x20;
}

uint32 Block::RenderHeader(IOCallback & output, Adler32 & checksum) const
{
    binary Header[BLOCK_HEADER_SIZE];

    // block size
    big_uint32 buf_size = size();
    buf_size.Fill(Header);

    /// timecode : it could also be big_uint32.Fill(&Header[2], Timecode)
    big_uint32 timecode = TimeCode;
    timecode.Fill(&Header[0x04]);

    // track number
    Header[0x08] = TrackNumber;

    uint8 flags = 0;
    flags |= (!KeyFrame << 6);
    flags |= (BFrame << 5);
    Header[0x09] = flags;

    // write buffer
    output.writeFully(Header,BLOCK_HEADER_SIZE);
    checksum.Feed(Header,BLOCK_HEADER_SIZE);

    return BLOCK_HEADER_SIZE;
}

bool Block::GetHeaderInfo(const binary * aBuffer, uint32 & aBufferSize, uint8 & aTrackNumber)
{
    // track number
    aTrackNumber = aBuffer[0x08];

    // block size
    big_uint32 block_size;
    block_size.Eval(&aBuffer[0x00]);
    uint32 aSize = block_size;
    
    if (aSize > aBufferSize)
	return false;

    aBufferSize = aSize;
    return true;
}

END_MATROSKA_NAMESPACE
