/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: Block.hpp,v 1.28 2002/11/20 22:37:39 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_BLOCK_HPP
#define LIBMATROSKA_BLOCK_HPP

#include <memory>
#include <vector>

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"
#include "Adler32.hpp"

START_MATROSKA_NAMESPACE

static const uint8 BLOCK_HEADER_SIZE=0x0A;

class Frame;

/*!
    \class Block
    \brief Handle the block encapsulation of data frames
    \todo Handle GapSet, read, write, trailer
    \see http://matroska.sourceforge.net/specs/#blockh
*/
class Block {
    public:
/*	enum add_result {
	    BLOCK_ADD_DONE,              ///< The frame has been added, more frames can be added
	    BLOCK_ADD_COMPLETE,          ///< This block is full now
	    BLOCK_ADD_ERR_BLOCK_FULL,    ///< This block was already full
	    BLOCK_ADD_ERR_USE_OTHER,     ///< Use another block to store the frame
	    BLOCK_ADD_ERR_MEMORY_FULL,   ///< No more memory on device
	    BLOCK_ADD_ERR_BAD_FRAME_LEN, ///< Use another block to store the frame
	};*/
	
	Block();

	Block(const uint8 track_number, const uint32 timecode);

	/*!
	    Create a block from an existing buffer
	*/
	Block(const uint8 track_number, const binary * aFrame, const uint32 aFrameSize);

	~Block() {}

	/*!
	    \return the number of bytes written to file
	*/
	virtual uint32 Render(IOCallback & output, Adler32 & checksum) = 0;

	/*!
	    \return the number of bytes written to file
	*/
	uint32 RenderHeader(IOCallback & output, Adler32 & checksum) const;

	static bool GetHeaderInfo(const binary * aBuffer, uint32 & aBufferSize, uint8 & aTrackNumber);
	
	/*!
	    This should function should be called with new frames while it returns BLOCK_ADD_DONE
	    \note No data is moved/copied
	    \return wether we can add more data to the Block
	*/
	virtual bool AddFrame(const binary * buffer, const uint32 buffer_size,
			      const bool KeyFrame, const bool BFrame) = 0;

	virtual uint32 size() const = 0;

	virtual bool GetNextFrame(uint32 & aTimecode, const Frame * & aFrame,
			      bool & aKeyFrame, bool & aBFrame) = 0;
	virtual bool NoFrameLeft() const = 0;
	
    protected:
	uint8 TrackNumber;
	uint32 TimeCode;
	bool KeyFrame;
	bool BFrame;
	bool GapSet;
	uint32 mySize; ///< including the header

	enum framing_mode {
	    SINGLE_PER_BLOCK,     ///< one frame per block
	    FIXED_SIZE_PER_BLOCK, ///< all frames of the track have the same size
	    OCTET_LACING          ///< multiple frames in a block on a Byte boundary
	};

    private:
	binary Header[BLOCK_HEADER_SIZE];
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_BLOCK_HPP
