/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: BlockOcL.hpp,v 1.17 2002/11/20 22:37:39 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_BLOCK_OCTET_LACING_HPP
#define LIBMATROSKA_BLOCK_OCTET_LACING_HPP

#include "Block.hpp"

START_MATROSKA_NAMESPACE

class Frame; // no need to include it here

/*!
    \class BlockOcL
    \brief Handle the block encapsulation of data frames, octet lacing used
*/
class BlockOcL : public Block {
    public:
	BlockOcL(const uint8 track_number, const uint32 timecode) :Block(track_number, timecode)
	{
	    FrameListIdx = FrameList.begin();
	}

	BlockOcL(const uint8 track_number, const binary * aFrame, const uint32 aFrameSize);
	/*!
	    Construct a Block by reading from the IO class
	*/
	BlockOcL(IOCallback & input);
	~BlockOcL();

	bool AddFrame(const binary * buffer, const uint32 buffer_size,
		      const bool KeyFrame, const bool BFrame);

	uint32 Render(IOCallback & output, Adler32 & checksum);

	inline uint32 size() const;

	bool GetNextFrame(uint32 & aTimecode, const Frame * & aFrame,
			      bool & aKeyFrame, bool & aBFrame);
	bool NoFrameLeft() const { return FrameListIdx == FrameList.end() ; }

    protected:
	std::vector<Frame*> FrameList;
	std::vector<Frame*>::const_iterator FrameListIdx;

	static const uint16 MAX_FRAME_SIZE_IN_LACE;
	static const uint8  MAX_LACES_PER_FRAME;

	inline uint32 all_size(uint32 & data_size, uint32 & lace_size) const;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_BLOCK_OCTET_LACING_HPP
