/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: BlockSPB.cpp,v 1.21 2002/11/20 22:37:39 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "Frame.hpp"
#include "BlockSPB.hpp"

#include <sstream>

using namespace std;

START_MATROSKA_NAMESPACE

BlockSPB::BlockSPB(const uint8 track_number, const uint32 timecode)
 :Block(track_number, timecode)
 ,myFrame(NULL)
{}

BlockSPB::BlockSPB(const uint8 track_number, const binary * aFrame, const uint32 aFrameSize)
 :Block(track_number, aFrame, aFrameSize)
 ,myFrame(NULL)
{
    aFrame += BLOCK_HEADER_SIZE;
    AddFrame(aFrame, aFrameSize - BLOCK_HEADER_SIZE, KeyFrame, BFrame);
}

/*!
    \return wether we should render the current block or not
*/
bool BlockSPB::AddFrame(const binary * buffer, const uint32 buffer_size, const bool a_KeyFrame, const bool a_BFrame)
{
    if (buffer == NULL)
    {
	stringstream Msg;
	Msg << "The buffer you want to write is NULL";
	throw runtime_error(Msg.str());
    }
    
    if (buffer_size == 0)
    {
	stringstream Msg;
	Msg << "Cannot write a blank frame";
	throw runtime_error(Msg.str());
    }
    
    if (myFrame != NULL)
    {
	stringstream Msg;
	Msg << "Block is full";
	throw runtime_error(Msg.str());
    }

    // here we know that this frame can fit in this block
    try {
	myFrame = new Frame(buffer, buffer_size);
	myFrameRead = false;
    }
    catch(string & Ex)
    {
	throw Ex;
    }
    
    KeyFrame = a_KeyFrame;
    BFrame = a_BFrame;

    // this is the last frame that the block can hold
    return true;
}

/*!
    \todo return the number of bytes written
 */
uint32 BlockSPB::Render(IOCallback & output, Adler32 & checksum)
{
    // write header
    uint32 result = RenderHeader(output, checksum);

    // write buffer
    if (myFrame != NULL)
    {
        output.writeFully(myFrame->buf(),myFrame->length());
	checksum.Feed(myFrame->buf(),myFrame->length());
	result += size();

	// free the frame
	delete myFrame;
	myFrame = NULL;
    }

    return result;
}

bool BlockSPB::GetNextFrame(uint32 & aTimecode, const Frame * & aFrame,
			      bool & aKeyFrame, bool & aBFrame)
{
    if (myFrame == NULL || myFrameRead)
	return false;

    aFrame = myFrame;
	aTimecode = TimeCode;
	aKeyFrame = KeyFrame;
	aBFrame = BFrame;

    myFrameRead = true;

    return true;
}

END_MATROSKA_NAMESPACE
