/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: BlockSPB.hpp,v 1.16 2002/11/20 22:37:39 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_BLOCK_SPB_HPP
#define LIBMATROSKA_BLOCK_SPB_HPP

#include "Block.hpp"
#include "Frame.hpp"

START_MATROSKA_NAMESPACE

/*!
    \class BlockSPB
    \brief Handle the block encapsulation of data frames, only one frame per block
*/
class BlockSPB : public Block {
    public:
	BlockSPB(const uint8 track_number, const uint32 timecode);
	BlockSPB(const uint8 track_number, const binary * aFrame, const uint32 aFrameSize);
	
	bool AddFrame(const binary * buffer, const uint32 buffer_size,
		      const bool KeyFrame, const bool BFrame);

	uint32 Render(IOCallback & output, Adler32 & checksum);

	void ReadFrame(binary *aFillFrame, uint32 & aFrameSize);

	inline uint32 size() const {
	    if (myFrame == NULL)
	        return 0;
	    else
	        return BLOCK_HEADER_SIZE + myFrame->length();
	}

	bool GetNextFrame(uint32 & aTimecode, const Frame * & aFrame,
			      bool & aKeyFrame, bool & aBFrame);
	bool NoFrameLeft() const { return myFrameRead; }

    protected:
	Frame *myFrame;
	bool   myFrameRead;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_BLOCK_SPB_HPP
