/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: Cluster.hpp,v 1.18 2002/05/23 16:47:12 robux4 Exp $
    \author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_CLUSTER_HPP
#define LIBMATROSKA_CLUSTER_HPP

#include <vector>

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"

START_MATROSKA_NAMESPACE

class Block; // no need to include it here

static const uint8 CLUSTER_HEADER_SIZE =0x10;
static const uint8 CLUSTER_TRAILER_SIZE=0x04;

/*!
	\class Cluster
	\brief container of Block classes
*/
class Cluster {
    public:
/*	enum add_result {
	    CLUSTER_ADD_DONE,           ///< Cool, everything worked fine
	    CLUSTER_ADD_COMPLETE,       ///< data added, but no more addition is possible
	    CLUSTER_ADD_ERR_BLOCK_FULL, ///< the Cluster is full, it should be rendered
	    CLUSTER_ADD_ERR_MEMORY_FULL,///< no more memory on device
	};*/

	/*!
	    Construct a Block from scratch
	 */
	Cluster(const uint32 max_size)
	    :MAX_SIZE(max_size)
	    ,mySize(0)
	    ,myBuffer(NULL)
	    ,myBufferBlockSize(0)
	    ,myBufferBlockPointer(NULL)
	{
	    Flush(); // just to make sure it's correctly initialised
	}

	Cluster::~Cluster();

	/*!
	    Add a block to the cluster
	    \return wether we can add more blocks or not
	*/
	bool AddBlock(Block * a_Block);

	/*!
	    Add a block to the cluster
	    \return wether there is a block remaining
	*/
	bool GetBlock(binary * & aBuffer, uint32 & aBufferSize, uint8 & aTrackNumber);

	/*!
	    \return the number of bytes written to file
	*/
	uint32 Render(IOCallback & output, uint32 & aNumberBlock) const;

	/*!
	    \return the offset were the cluster head was found (0 when normal)
	*/
	uint32 FindHead(IOCallback & input);

	/*!
	    \return The size of all the data stored in the cluster
	 */
	inline uint32 size() const;

	inline void setMaxSize(const uint32 value) {MAX_SIZE = value;}

	inline bool BlockLeft() const {
	    return myBufferBlockSize != 0; 
	}

	void Flush();

    protected:
	static const character HEADER[4];
    
	std::vector<Block*>  myBlockList;

	uint32 MAX_SIZE;
	uint32 mySize; ///< total cluster size, including header and footer
	binary * myBuffer;
	binary * myBufferBlockPointer;
	uint32 myBufferBlockSize;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_CLUSTER_HPP
