/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: CodecHeader.cpp,v 1.2 2002/05/24 15:53:41 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "CodecHeader.hpp"
#include "Track.hpp"

START_MATROSKA_NAMESPACE

const character CodecHeader::HEADER[4] = { 'C', 'o', 'd', 'H' };

uint32 CodecHeader::Render(IOCallback & output, const std::vector<Track *> & aTracks)
{
    uint32 result = 0;

    output.writeFully(HEADER,4);
    result += 4;

    for (unsigned int i=0; i<aTracks.size(); i++)
    {
	result += aTracks[i]->RenderCodecHeader(output);
    }

    return result;
}

uint32 CodecHeader::Read(IOCallback & input, std::vector<Track *> & aTracks)
{
    uint32 result = 0;

    binary Head[4];
    input.readFully(Head,4);
    result += 4;

    if (memcmp(Head, HEADER, 4) != 0)
	throw 0; // warning not a codec header

    for (unsigned int i=0; i<aTracks.size(); i++)
    {
	result += aTracks[i]->ReadCodecHeader(input);
    }

    return result;
}

END_MATROSKA_NAMESPACE
