/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: ContentInfo.hpp,v 1.8 2002/05/08 09:12:45 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_CONTENT_INFO_HPP
#define LIBMATROSKA_CONTENT_INFO_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"
#include "TruncString.hpp"

START_MATROSKA_NAMESPACE

/*!
	\class ContentInfo
	\brief User-definable meta informations about the current Matroska file
	\see http://matroska.sourceforge.net/specs/#extended2
*/
class ContentInfo {
    public:
	ContentInfo();

	uint32 Render(IOCallback & output) const;

	/*!
	    \return the number of bytes read from file
	*/
	uint32 Read(IOCallback & input);

	static inline uint32 default_size() {return 1024;}

    protected:
	TruncString8 myTextFields1;
	TruncString8 myTextFields2;
	TruncString8 myTextFields3;
	TruncString8 myTextFields4;
	TruncString8 myTextFields5;
	TruncString8 myTextFields6;
	TruncString8 myTextFields7;
	TruncString8 mySmallText;
	bool   myCheckBoxes[8];
	uint8  myRadioButton1;
	uint8  myRadioButton2;
	uint8  myContentSubtype;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_CONTENT_INFO_HPP
