/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: ExtendedInfo.hpp,v 1.8 2002/05/06 20:45:25 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_EXTENDED_INFO_HPP
#define LIBMATROSKA_EXTENDED_INFO_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"
#include "TruncString.hpp"

START_MATROSKA_NAMESPACE

static const size_t RSACI_SIZE = 6;

/*!
	\class ExtendedInfo
	\brief Meta informations about the current Matroska file
	\see http://matroska.sourceforge.net/specs/#extended
*/
class ExtendedInfo {
    public:
	ExtendedInfo();

	uint32 Render(IOCallback & output) const;

	/*!
	    \return the number of bytes read from file
	*/
	uint32 Read(IOCallback & input);

	static inline uint32 default_size() {return 3072;}

    protected:
	static const size_t TITLE_SIZE;
	TruncString8 myTitle;

	static const size_t EDITION_SIZE;
	TruncString8 myEdition;

	static const size_t NAME_SIZE;
	TruncString8 myName;

	static const size_t URL_SIZE;
	TruncString myURL;

	static const size_t EMAIL_SIZE;
	TruncString myEmail;

	static const size_t ENCODER_NAME_SIZE;
	TruncString8 myEncoderName;

	static const size_t ENCODER_URL_SIZE;
	TruncString myEncoderURL;

	static const size_t ENCODER_EMAIL_SIZE;
	TruncString myEncoderEmail;

	static const size_t DESCRIPTION_SIZE;
	TruncString8 myDescription;

	static const size_t COUNTRY_CODE_SIZE;
	TruncString myCountryCode;

//	static const size_t LANGUAGE_CODE_SIZE;
//	TruncString myLanguageCode;
	binary myLanguageCode[4]; ///< \see http://matroska.sourceforge.net/language.htm
    
	binary myRsaci[RSACI_SIZE]; ///< \see http://matroska.sourceforge.net/rsaci.htm

	uint8 myContentType; ///< \todo move to a clas with the strings
	timecode myEncodingDate;
	timecode myLastEditingDate;
	uint16 myProductionYear;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_EXTENDED_INFO_HPP
