/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: FileName.cpp,v 1.3 2002/05/23 10:12:10 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "FileName.hpp"

START_MATROSKA_NAMESPACE

FileName::FileName(size_t limit, const std::string & str, const char a_pad)
 :TruncString(limit, str, left, a_pad)
{}

void FileName::MakeSafe()
{
    std::string tmpString = "";
    char tmpChar;
    for (unsigned int i=0; i<myString.size(); i++)
    {
	tmpChar = myString[i];
	if ((tmpChar <= 'Z' && tmpChar >= 'A') ||
	    (tmpChar <= 'z' && tmpChar >= 'a') ||
	    (tmpChar <= '9' && tmpChar >= '0') ||
	    tmpChar == '_' ||
	    tmpChar == '-' ||
	    tmpChar == '(' ||
	    tmpChar == ')')
	{
	    tmpString += tmpChar;
	}
	else if (tmpChar == '.')
	{
	    if ((tmpString.size() != 0) && tmpString[tmpString.size()-1] != '.' && (i+1 != myString.size()))
		tmpString += tmpChar;
	}
    }

    Fill(tmpString);
}

void FileName::MakeRelaxed()
{
    std::string tmpString = "";
    char tmpChar;
    for (unsigned int i=0; i<myString.size(); i++)
    {
	tmpChar = myString[i];
	if ((tmpChar <= 'Z' && tmpChar >= 'A') ||
	    (tmpChar <= 'z' && tmpChar >= 'a') ||
	    (tmpChar <= '9' && tmpChar >= '0') ||
	    tmpChar == '_' ||
	    tmpChar == '-' ||
	    tmpChar == '(' ||
	    tmpChar == ')')
	{
	    tmpString += tmpChar;
	}
	else if (tmpChar == '.')
	{
	    if ((tmpString.size() != 0) && tmpString[tmpString.size()-1] != '.' && (i+1 != myString.size()))
		tmpString += tmpChar;
	}
	else if (tmpChar == ' ')
	{
	    if ((tmpString.size() != 0) && tmpString[tmpString.size()-1] != ' ' && (i+1 != myString.size()))
		tmpString += tmpChar;
	}
    }

    Fill(tmpString);
}

FileName::name_level FileName::Test() const
{
    bool bRelaxed = false;
    bool bValid = true;

    char tmpChar;
    for (unsigned int i=0; i<myLimit; i ++)
    {
	tmpChar = myBuffer[i];
	if (tmpChar == '.')
	{
	    if (i == 0 || i == myLimit || myBuffer[i+1] == '\0') {
		bValid = false;
		break;
	    } else if (myBuffer[i-1] == '.')
		bRelaxed = true;
	}
	if (tmpChar == ' ')
	{
	    if (i == 0 || i == myLimit || myBuffer[i+1] == '\0') {
		bValid = false;
		break;
	    } else bRelaxed = true;
	}
	if ((tmpChar > 'Z' && tmpChar < 'A') &&
	    (tmpChar < 'z' && tmpChar < 'a') &&
	    (tmpChar > '9' && tmpChar < '0') &&
	    tmpChar != '_' &&
	    tmpChar != '-' &&
	    tmpChar != '(' &&
	    tmpChar != ')')
	{
	    bValid = false;
	    break;
	}
    }

    return (!bValid ? NAME_INVALID: (bRelaxed ? NAME_RELAXED: NAME_VALID));
}

END_MATROSKA_NAMESPACE
