/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: Frame.cpp,v 1.5 2002/04/29 19:27:45 robux4 Exp $
	\note the Frame bufferize the data in another buffer, there should be a UnbufferedFrame object later for low memory systems
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include "StdInclude.hpp"
#include "Frame.hpp"

#include <sstream>

using namespace std;

START_MATROSKA_NAMESPACE

Frame::Frame(const binary * buffer, const uint32 buffer_size)
 :BufferSize(buffer_size)
{
    if (buffer==NULL)
    {
	stringstream Msg;
	Msg << "Frame created with null buffer";
	throw runtime_error(Msg.str());
    }
    if (buffer_size==0)
    {
	stringstream Msg;
	Msg << "Frame created with no data";
	throw runtime_error(Msg.str());
    }
    try {
	Buffer = new binary[buffer_size];
	memcpy(Buffer, buffer, buffer_size);
    }
    catch (...)
    {
	stringstream Msg;
	Msg << "No more memory for frame";
	throw runtime_error(Msg.str());
    }
}

Frame::~Frame() {
    if (Buffer != NULL)
	delete Buffer;
}

END_MATROSKA_NAMESPACE
