/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Ingo Ralf Blum

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: KaxError.cpp,v 1.4 2002/04/14 10:03:01 robux4 Exp $
*/
#include "StdInclude.hpp"
#include "KaxError.hpp"
#include "Config.hpp"

START_MATROSKA_NAMESPACE

const char *KaxError::getMessage(error Error)throw()
{
	switch (Error) {
	case error_no_error:
		return "None";
	case error_out_of_memory:
		return "Insufficient memory";
	case error_too_many_warnings:
		return "Too many warnings in buffer";
	case error_cant_open_file:
		return "Cannot open file";
	case error_cant_close_file:
		return "Cannot close file";
	case error_cant_read_main_header:
		return "Unable to read MAIN HEADER (too small file?)";
	case error_matroska_header_missing:
		return "This is not a valid Matroska-file - header is missing";
	case error_file_size_mismatch:
		return "File size in type-header is different to size of the actual file";
	case error_read_version_mismatch:
		return "Absolute minimum read-version is greater than our version, upgrade your Matroska-library";
	case error_invalid_actual_header_checksum:
		return "Checksum of ACTUAL HEADER is invalid";
	case error_invalid_extended_info_checksum:
		return "Checksum of EXTENDED INFO is invalid";
	case error_invalid_content_spec_info_checksum:
		return "Checksum of CONTENT-SPECIFIC INFO is invalid";
	case error_cant_read_track_entry:
		return "Unable to read TRACK-ENTRY (incorrect data or too small file?)";
	case error_invalid_track_entry_header:
		return "Header of a TRACK-ENTRY missing/invalid (broken file?)";
	case error_invalid_invalid_track_entry_checksum:
		return "Checksum of a TRACK-ENTRY is invalid";
	case error_track_number_mismatch:
		return
			"Track-number in a TRACK-ENTRY doesn't match to actual number of that track (broken file?)";
	case error_cant_read_block_entry:
		return "Unable to read BLOCK-ENTRY (incorrect data or too small file?)";
	case error_insequential_timecodes:
		return "Insequential timecodes (in block-entries)";
	case error_invalid_block_entry_checksum:
		return "Checksum of BLOCK-ENTRIES is invalid";
	case error_no_suitable_blocks_for_seek:
		return "Seek failed - no suitable blocks found";
	case error_max_track_count_reached:
		return "Cannot add another track - maximum of 255 tracks reached";
	case error_cant_write_main_header:
		return "Unable to write MAIN HEADER (disk full?)";
	case error_cant_write_track_entry:
		return "Unable to write TRACK-ENTRY (disk full?)";
	case error_cant_write_codec_header:
		return "Unable to write CODEC-HEADER (disk full?)";
	case error_cant_write_block:
		return "Unable to write BLOCK (disk full?)";
	case error_cant_write_checksum:
		return "Unable to write CHECKSUM (disk full?)";
	case error_cant_write_block_entry:
		return "Unable to write BLOCK-ENTRY (disk full?)";
	case error_cant_write_signature:
		return "Unable to write SIGNATURE-HEADER (disk full?)";
	case error_cant_read_back_block_header:
		return "Unable to read back BLOCK-HEADER";
	case error_minimum_read_version_greater_than_ours:
		return "Minimum read-version is greater than our version - we can't use all features";
	}
	return "Unknown error (this is a bug)";
}

END_MATROSKA_NAMESPACE