/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Ingo Ralf Blum

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: KaxError.hpp,v 1.5 2002/05/17 18:02:18 robux4 Exp $
*/
#ifndef KAX_ERROR_HPP
#define KAX_ERROR_HPP

#include "Config.hpp"

#include <stdexcept>
#include <string>


START_MATROSKA_NAMESPACE

enum error
{
	error_no_error,
	error_out_of_memory,
	error_too_many_warnings,
	error_cant_open_file,
	error_cant_close_file,
	error_cant_read_main_header,
	error_matroska_header_missing,
	error_file_size_mismatch,
	error_read_version_mismatch,
	error_invalid_actual_header_checksum,
	error_invalid_extended_info_checksum,
	error_invalid_content_spec_info_checksum,
	error_cant_read_track_entry,
	error_invalid_track_entry_header,
	error_invalid_invalid_track_entry_checksum,
	error_track_number_mismatch,
	error_cant_read_block_entry,
	error_insequential_timecodes,
	error_invalid_block_entry_checksum,
	error_no_suitable_blocks_for_seek,
	error_max_track_count_reached,
	error_cant_write_main_header,
	error_cant_write_track_entry,
	error_cant_write_codec_header,
	error_cant_write_block,
	error_cant_write_checksum,
	error_cant_write_block_entry,
	error_cant_write_signature,
	error_cant_read_back_block_header,
	error_minimum_read_version_greater_than_ours
};

class MatroskaError:public std::runtime_error
{
private:
	error Error;

public:
	static const char*getMessage(error Error)throw();

	MatroskaError(error Error,const std::string&Description)
		:std::runtime_error(Description+": "+getMessage(Error))
		,Error(Error)
	{
	}

	MatroskaError(error Error)
		:std::runtime_error(std::string("Unspecified error: ")+getMessage(Error))
		,Error(Error)
	{
	}

	error getError()const throw(){return Error;}
};

END_MATROSKA_NAMESPACE

#endif // KAX_ERROR_HPP
