/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: MainHeader.cpp,v 1.9 2002/05/09 10:07:34 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "MainHeader.hpp"

START_MATROSKA_NAMESPACE

MainHeader::MainHeader()
:myActualHeader()
{}

/*!
    \todo Handle aTrackSize differently
*/
uint32 MainHeader::Render(IOCallback & output, const StreamInfo & aStreamInfo) const
{
    uint32 size_written = 0;

    size_written += myType.Render(output);
    size_written += myActualHeader.Render(output
//	                                  ,aTrackSize
//					  ,myType.default_size() + myActualHeader.default_size() +
//					  myExtendedInfo.default_size() + myContentInfo.default_size()
					  ,aStreamInfo);
    size_written += myExtendedInfo.Render(output);
    size_written += myContentInfo.Render(output);

    return size_written;
}

uint32 MainHeader::Read(IOCallback & input, StreamInfo & aStreamInfo)
{
    uint32 size_read = 0;
    
    try {
	size_read += myType.Read(input);
    }
    catch (exception & Ex)
    {
	throw Ex;
    }

    // if this one fail, the file is unreadable
    try {
//	size_read += myActualHeader.Read(input, aNumberBlock, aTrackEntryPosition);
	size_read += myActualHeader.Read(input, aStreamInfo);
    }
    catch (exception & Ex)
    {
	throw Ex;
    }

    try {
	size_read += myExtendedInfo.Read(input);
	size_read += myContentInfo.Read(input);
    }
    catch (exception & Ex)
    {
	throw Ex;
    }

    return size_read;
}

END_MATROSKA_NAMESPACE
