/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: StreamInfo.hpp,v 1.3 2002/05/23 10:12:10 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_STREAM_INFO_HPP
#define LIBMATROSKA_STREAM_INFO_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "TruncString.hpp"

START_MATROSKA_NAMESPACE

/*!
    \class FileInfo
    \brief
*/
class StreamInfo {
    public:
	StreamInfo()
	    :TimeLength(0)
	    ,MainHeaderSize(0)
	    ,ClusterPosition(0)
	    ,ClusterSize(0)
	    ,TrackEntryPosition(0)
	    ,TrackEntriesSize(0)
	    ,CodecEntryPosition(0)
	    ,CodecEntrySize(0)
	    ,SeekEntryPosition(0)
	    ,SeekEntriesSize(0)
	    ,ChapterEntryPosition(0)
	    ,ChapterEntrySize(0)
	    ,AttachEntryPosition(0)
	    ,AttachEntrySize(0)
	    ,SignatureEntryPosition(0)
	    ,SignatureEntrySize(0)
	    ,NumberBlock(0)
	    ,ClusterHeadSize(0)
	    ,BlockHeadSize(0)
	    ,ClusterFootSize(0)
	    ,SeekEntrySize(0)
	    ,TrackEntrySize(0)
	    ,EncoderLib(20)
	    ,EncoderApp(24)
	{}

	uint32 TimeLength;
	uint32 MainHeaderSize;

	uint64 ClusterPosition;
	uint64 ClusterSize;

	uint64 TrackEntryPosition;
	uint32 TrackEntriesSize;

	uint64 CodecEntryPosition;
	uint32 CodecEntrySize;

	uint64 SeekEntryPosition;
	uint32 SeekEntriesSize;

	uint64 ChapterEntryPosition;
	uint32 ChapterEntrySize;

	uint64 AttachEntryPosition;
	uint32 AttachEntrySize;

	uint64 SignatureEntryPosition;
	uint32 SignatureEntrySize;

	uint32 NumberBlock;

	// element sizes
	uint8 ClusterHeadSize;
	uint8 BlockHeadSize;
	uint8 ClusterFootSize;
	uint8 SeekEntrySize;
	uint16 TrackEntrySize;

	TruncString8 EncoderLib;
	TruncString8 EncoderApp;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_STREAM_INFO_HPP
