/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TrackAudio.cpp,v 1.7 2002/06/17 20:06:44 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "TrackAudio.hpp"

START_MATROSKA_NAMESPACE

TrackAudio::TrackAudio()
 :TrackType(track_audio)
{
    myInfo.SampleRate=0;
    myInfo.BitDepth=0;
    myInfo.DTS=false;
    myInfo.SecondaryTrack=false;
    myInfo.SubtrackNumber=0;
    myInfo.ChannelPerSubtrack=0;
    myInfo.AVIFormatTag=0;
    myInfo.ChannelPosition=0;
}

uint32 TrackAudio::Render(binary *buffer) const
{
    memset(buffer, 0, default_size());
    binary *cursor = buffer;

    // sample rate
    big_uint32 _sample = myInfo.SampleRate;
    _sample.Fill(cursor);
    cursor += 4;

    // bit depth
    *cursor++ = myInfo.BitDepth;

    // flags
    *cursor++ = (myInfo.DTS ? 0x01:0) | (myInfo.SecondaryTrack ? 0x02:0);

    // number of subtracks
    *cursor++ = myInfo.SubtrackNumber;

    // number of channels per subtrack
    *cursor++ = myInfo.ChannelPerSubtrack;

    // sample rate
    big_uint16 _FormatTag = myInfo.AVIFormatTag;
    _FormatTag.Fill(cursor);
    cursor += 2;

    // channel positionning
    big_uint32 _Position = myInfo.ChannelPosition;
    _Position.Fill(cursor);
    cursor += 4;

    return default_size();
}

void TrackAudio::Read(const binary *buffer)
{
    // sample rate
    big_uint32 bu32;
    bu32.Eval(buffer);
    buffer += sizeof(uint32);
    myInfo.SampleRate = bu32;

    // bit depth
    myInfo.BitDepth = *buffer++;

    // flags
    myInfo.DTS = (*buffer & 0x01) != 0;
    myInfo.SecondaryTrack = (*buffer & 0x02) != 0;
    buffer++;

    // subtracks
    myInfo.SubtrackNumber = *buffer++;

    // ChannelPerSubtrack
    myInfo.ChannelPerSubtrack = *buffer++;

    // AVI compatibility
    big_uint16 bu16;
    bu16.Eval(buffer);
    buffer += sizeof(uint16);
    myInfo.AVIFormatTag = bu16;

    // Channel position
    bu32.Eval(buffer);
    myInfo.ChannelPosition = bu32;
}

void TrackAudio::SetInfo(const TrackInfoAudio & aTrackInfo)
{
    myInfo = aTrackInfo;
}

END_MATROSKA_NAMESPACE
