/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: TrackSubtitle.cpp,v 1.4 2002/05/13 21:26:47 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "TrackSubtitle.hpp"

START_MATROSKA_NAMESPACE

TrackSubtitle::TrackSubtitle()
 :TrackType(track_subtitle)
 ,myHeaderSize(4) // default for the moment
 ,myColor1(0,0,0)
 ,myColor2(0,0,0)
 ,myColor3(0,0,0)
 ,myColor4(0,0,0)
{
    myInfo.myHas_1 = false;
    myInfo.myControlCodeEnabled = true;
    myInfo.myCharSubset = 0; // UTF-8
}

uint32 TrackSubtitle::Render(binary *buffer) const
{
    memset(buffer, 0, default_size());
    binary *cursor = buffer;

    // Flags
    *cursor++ = (myInfo.myHas_1 ? 0x01:0) | (myInfo.myControlCodeEnabled ? 0:0x02);

    // Charset
    *cursor++ = myInfo.myCharSubset;

    // Header size
    big_uint16 _HeaderSize = myHeaderSize;
    _HeaderSize.Fill(cursor);
    cursor += 2;

    // Colors
    myColor1.Fill(cursor);
    cursor += myColor1.size();

    myColor2.Fill(cursor);
    cursor += myColor2.size();

    myColor3.Fill(cursor);
    cursor += myColor3.size();

    myColor4.Fill(cursor);
    cursor += myColor4.size();

    return default_size();
}

void TrackSubtitle::Read(const binary *buffer)
{
    // Flags
    myInfo.myHas_1 = (*buffer & 0x01) != 0;
    myInfo.myControlCodeEnabled = (*buffer & 0x02) == 0;
    buffer++;

    // charset
    myInfo.myCharSubset = *buffer++;

    // subtitle-header size
    big_uint16 bu16;
    bu16.Eval(buffer);
    buffer += sizeof(uint16);
    myHeaderSize = bu16;

    // Colors
    myColor1.Eval(buffer);
    buffer += myColor1.size();

    myColor2.Eval(buffer);
    buffer += myColor1.size();

    myColor3.Eval(buffer);
    buffer += myColor1.size();

    myColor4.Eval(buffer);
    buffer += myColor1.size();
}

END_MATROSKA_NAMESPACE
