/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TrackSubtitle.hpp,v 1.5 2002/05/13 21:26:47 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_TRACK_SUBTITLE_HPP
#define LIBMATROSKA_TRACK_SUBTITLE_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "TrackType.hpp"
#include "IOCallback.hpp"

START_MATROSKA_NAMESPACE

class RGB_col {
    public:
	RGB_col(uint8 _red, uint8 _green, uint8 _blue)
	    :red(_red), green(_green), blue(_blue) {}

	uint8 red;
	uint8 green;
	uint8 blue;

	inline void Fill(binary *buffer) const
	{
	    *buffer++ = red;
	    *buffer++ = green;
	    *buffer++ = blue;
	}

	inline void Eval(const binary *buffer)
	{
	    red = *buffer++;
	    green = *buffer++;
	    blue = *buffer++;
	}

	static inline size_t size() {return 3;}
};

/*!
	\class TrackSubtitle
	\brief
*/
class TrackSubtitle : public TrackType {
    public:
	TrackSubtitle();

	uint32 Render(binary *buffer) const;
	void Read(const binary *buffer);

    protected:
	uint16            myHeaderSize;
	TrackInfoSubtitle myInfo;
	RGB_col           myColor1;
	RGB_col           myColor2;
	RGB_col           myColor3;
	RGB_col           myColor4;
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_TRACK_SUBTITLE_HPP
