/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TrackType.cpp,v 1.5 2002/06/17 20:06:44 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "TrackType.hpp"
#include "TrackAudio.hpp"
#include "TrackAV.hpp"
#include "TrackControl.hpp"
#include "TrackCustom.hpp"
#include "TrackMultiAudio.hpp"
#include "TrackSubtitle.hpp"
#include "TrackTitle.hpp"
#include "TrackVideo.hpp"

START_MATROSKA_NAMESPACE

TrackType * TrackType::GetNewTrack(const track_type aType)
{
    TrackType *result = NULL;
    switch (aType)
    {
/*	case track_control:
	result = new TrackControl;
	break;
*/    case track_video:
	result = new TrackVideo;
	break;
	case track_audio:
	result = new TrackAudio;
	break;
/*	case track_av:
	result = new TrackAV;
	break;
    case track_multi_audio:
	result = new TrackMultiAudio;
	break;
    case track_title:
	result = new TrackTitle;
	break;
*/	case track_subtitle:
	result = new TrackSubtitle;
	break;
/*	case track_custom:
	result = new TrackCustom;
	break;
*/	default:
	throw 0; // track-type unknown
    }
    return result;
}

void TrackType::GetInfo(void * aTrackInfo, const size_t aInfoSize) const
{
    switch (myType)
    {
/*	case track_control:
	result = new TrackControl;
	break;
*/    case track_video:
	if (sizeof(TrackInfoVideo) != aInfoSize)
	    throw 0;
	*(TrackInfoVideo*)aTrackInfo = *reinterpret_cast<const TrackVideo*>(this);
	break;
	case track_audio:
	if (sizeof(TrackInfoAudio) != aInfoSize)
	    throw 0;
	*(TrackInfoAudio*)aTrackInfo = *reinterpret_cast<const TrackAudio*>(this);
	break;
/*	case track_av:
	result = new TrackAV;
	break;
    case track_multi_audio:
	result = new TrackMultiAudio;
	break;
    case track_title:
	result = new TrackTitle;
	break;
* /	case track_subtitle:
	if (sizeof(TrackInfoSubtitle) != aInfoSize)
	    throw 0;
	*(TrackInfoSubtitle*)aTrackInfo = TrackInfoSubtitle(*reinterpret_cast<const TrackSubtitle*>(this));
	break;
/*	case track_custom:
	result = new TrackCustom;
	break;
*/	default:
	throw 0; // track-type unknown
    }
}

END_MATROSKA_NAMESPACE
