/*
    libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
    Copyright (C) 2002 Steve Lhomme

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TrackType.hpp,v 1.5 2002/05/09 21:06:06 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_TRACK_TYPE_HPP
#define LIBMATROSKA_TRACK_TYPE_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"

START_MATROSKA_NAMESPACE

/*!
    \class TrackType
    \brief
    \see http://matroska.sourceforge.net/trackt.htm
*/
class TrackType {
    public:
	static inline uint32 default_size() {return 0x240-0x180-0x04;}

	TrackType(track_type a_type) :myType(a_type) {}

	operator track_type() const {return myType;}

	virtual uint32 Render(binary *buffer) const = 0;
	virtual void Read(const binary *buffer) = 0;

	static TrackType * GetNewTrack(const track_type aType);

	void GetInfo(void * aTrackInfo, const size_t aInfoSize) const;

    protected:
	track_type myType;

};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_TRACK_TYPE_HPP
