/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TrackVideo.cpp,v 1.4 2002/06/17 20:06:44 robux4 Exp $
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "TrackVideo.hpp"

START_MATROSKA_NAMESPACE

TrackVideo::TrackVideo()
 :TrackType(track_video)
{
    myInfo.xSize = 0;
    myInfo.ySize = 0;
    myInfo.FourCC[0] = 0;
    myInfo.FourCC[1] = 0;
    myInfo.FourCC[2] = 0;
    myInfo.FourCC[3] = 0;

    for (int i=0; i<16 ;i ++)
	myInfo.GUID[i] = 0;
}

uint32 TrackVideo::Render(binary *buffer) const
{
    memset(buffer, 0, default_size());
    binary *cursor = buffer;

    // flags
    *cursor++ = (myInfo.Interlaced ? 0x01:0) | (myInfo.Stereo3D ? 0x02:0) | ((uint8(myInfo.Eyes) & 0x03) << 2);
    cursor++;

    // dimensions
    big_uint16 bu16 = myInfo.xSize;
    bu16.Fill(cursor);
    cursor += sizeof(uint16);

    bu16 = myInfo.ySize;
    bu16.Fill(cursor);
    cursor += sizeof(uint16);

    // aspect ratio
    bu16 = myInfo.AspectRatio;
    bu16.Fill(cursor);
    cursor += sizeof(uint16);

    // FourCC
    *cursor++ = myInfo.FourCC[0];
    *cursor++ = myInfo.FourCC[1];
    *cursor++ = myInfo.FourCC[2];
    *cursor++ = myInfo.FourCC[3];

    // colour space
    big_uint32 bu32 = myInfo.ColourSpace;
    bu32.Fill(cursor);
    cursor += sizeof(uint32);

    // GUID
    for (int i=0; i<16 ;i ++)
	*cursor++ = myInfo.GUID[i];

    // gamma
    *cursor++ = myInfo.Gamma;

    return default_size();
}

void TrackVideo::Read(const binary *buffer)
{
    // Flags
    myInfo.Interlaced = (*buffer & 0x01) != 0;
    myInfo.Stereo3D   = (*buffer & 0x02) != 0;
    myInfo.Eyes       = VideoStereoEye((*buffer >> 2) & 0x03);
    buffer++;

    buffer++;

    // dimensions
    big_uint16 bu16;
    bu16.Eval(buffer);
    buffer += sizeof(uint16);
    myInfo.xSize = bu16;

    bu16.Eval(buffer);
    buffer += sizeof(uint16);
    myInfo.ySize = bu16;

    // aspect ratio
    bu16.Eval(buffer);
    buffer += sizeof(uint16);
    myInfo.AspectRatio = bu16;

    // FourCC
    myInfo.FourCC[0] = *buffer++;
    myInfo.FourCC[1] = *buffer++;
    myInfo.FourCC[2] = *buffer++;
    myInfo.FourCC[3] = *buffer++;

    // colour space
    big_uint32 bu32;
    bu32.Eval(buffer);
    buffer += sizeof(uint32);
    myInfo.ColourSpace = bu32;

    // GUID
    for (int i=0; i<16 ;i ++)
	myInfo.GUID[i] = *buffer++;

    // gamma
    myInfo.Gamma = *buffer++;
}

void TrackVideo::SetInfo(const TrackInfoVideo & aTrackInfo)
{
    myInfo = aTrackInfo;
}

END_MATROSKA_NAMESPACE
