/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TruncString.cpp,v 1.4 2002/05/03 19:22:31 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include <algorithm>

#include "StdInclude.hpp"
#include "TruncString.hpp"

START_MATROSKA_NAMESPACE

using namespace std;

TruncString::TruncString(size_t limit, const string & str, const align a_align, const char a_pad)
 :myString(str)
 ,myLimit(limit)
 ,myBuffer(NULL)
 ,myAlignement(a_align)
 ,myPad(a_pad)
{
    try {
    	Fill(str);
    }
    catch(exception & Ex)
    {
	throw Ex;
    }
}

TruncString::~TruncString()
{
    if (myBuffer != NULL)
	delete myBuffer;
}

void TruncString::Fill(const string & aStr)
{
    if (myBuffer == NULL)
    try
    {
        myBuffer = new binary[myLimit];
    } 
    catch (exception & Ex)
    {
	throw Ex;
    }

    size_t str_size = aStr.size();

    if (myLimit < str_size) {
	memcpy(myBuffer, aStr.c_str(), myLimit);
    } else {
	if (myAlignement == left)
	{
	    memcpy(myBuffer, aStr.c_str(), str_size);
	    memset(&myBuffer[str_size],myPad,myLimit - str_size);
	}
	else
	{
	    memcpy(&myBuffer[myLimit - str_size], aStr.c_str(), str_size);
	    memset(myBuffer,myPad,myLimit - str_size);
	}
    }
    myString = aStr;
}

TruncString & TruncString::operator=(const string & aStr)
{
    try {
    	Fill(aStr);
    }
    catch(exception & Ex)
    {
	throw Ex;
    }

    return *this;
}

TruncString8::TruncString8(size_t limit, const string & str, const align a_align, const char a_pad) 
 :myString("")
 ,myLimit(limit)
 ,myBuffer(NULL)
 ,myAlignement(a_align)
 ,myPad(a_pad)
{
    try {
    	Fill(str);
    }
    catch(exception & Ex)
    {
	throw Ex;
    }
}

TruncString8::~TruncString8()
{
    if (myBuffer != NULL)
	delete myBuffer;
}

void TruncString8::Fill(const string & aStr)
{
    if (myBuffer == NULL)
    try {
        myBuffer = new binary[myLimit];
    }
    catch (exception & Ex)
    {
	throw Ex;
    }

    size_t str_size = aStr.size();

    if (myLimit < str_size) {
	memcpy(myBuffer, aStr.c_str(), myLimit);
    } else {
	if (myAlignement == left)
	{
	    memcpy(myBuffer, aStr.c_str(), str_size);
	    memset(&myBuffer[str_size],myPad,myLimit - str_size);
	}
	else
	{
	    memcpy(&myBuffer[myLimit - str_size], aStr.c_str(), str_size);
	    memset(myBuffer,myPad,myLimit - str_size);
	}
    }
    myString = aStr;
}
/*
void TruncString8::Fill(const wstring & aStr)
{
    size_t str_size = aStr.size();

    mbstate_t state=0;
    const wchar_t *cursor = aStr.c_str();
    str_size = wcsrtombs((char*)myBuffer, &cursor, aStr.size(), &state);

    if (myLimit > str_size) {
	if (myAlignement == left)
	{
	    memset(&myBuffer[str_size],' ',myLimit - str_size);
	}
	else
	{
	    memset(myBuffer,' ',myLimit - str_size);
	}
    }
    myString = (char*)myBuffer;
}
*/
TruncString8 & TruncString8::operator=(const string & aStr)
{
    try {
	if (myBuffer != NULL)
	    delete myBuffer;

	myBuffer = new binary[myLimit];

    	Fill(aStr);
    }
    catch(exception & Ex)
    {
	throw Ex;
    }

    return *this;
}
/*
TruncString8 & TruncString8::operator=(const wstring & aStr)
{
    try {
	if (myBuffer != NULL)
	    delete myBuffer;

	myBuffer = new binary[myLimit];

    	Fill(aStr);
    }
    catch(exception & Ex)
    {
	throw Ex;
    }

    return *this;
}
*/
END_MATROSKA_NAMESPACE
