/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TypeHeader.cpp,v 1.8 2002/05/27 18:04:56 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.hpp"
#include "TypeHeader.hpp"

START_MATROSKA_NAMESPACE

const binary TypeHeader::HEAD[TYPE_HEADER_HEAD_SIZE] = {'M', 'C', 'F', ' ', '-', ' '};
const binary TypeHeader::SEPARATION_1[TYPE_HEADER_SEP1_SIZE] = {'\r', '\n', 'S', 'i', 't', 'e', ':', ' ', '['};
const binary TypeHeader::SEPARATION_2[TYPE_HEADER_SEP2_SIZE] = {']', '\r', '\n', 'S', 'i', 'z', 'e', ':', ' ', '['};
const binary TypeHeader::TAIL[TYPE_HEADER_TAIL_SIZE] = {']', '\r', '\n', 0x1A};

const size_t TypeHeader::INFO_SIZE = 0x31;
const size_t TypeHeader::ADS_SIZE  = 0x40;
const size_t TypeHeader::SIZE_SIZE = 0x12;

TypeHeader::TypeHeader()
 :myInfo(INFO_SIZE, "more info: http://matroska.sourceforge.net/",TruncString::left,' ')
 ,myAds(ADS_SIZE, "", TruncString::left,' ')
 ,mySize(SIZE_SIZE,"", TruncString::left,' ')
{}

uint32 TypeHeader::Render(IOCallback & output) const
{
    output.writeFully(HEAD,TYPE_HEADER_HEAD_SIZE);
    output.writeFully(myInfo.buffer(),myInfo.size());

    output.writeFully(SEPARATION_1,TYPE_HEADER_SEP1_SIZE);
    output.writeFully(myAds.buffer(),myAds.size());

    output.writeFully(SEPARATION_2,TYPE_HEADER_SEP2_SIZE);
    output.writeFully(mySize.buffer(),mySize.size());

    output.writeFully(TAIL,TYPE_HEADER_TAIL_SIZE);

    return default_size();
}

void TypeHeader::SetSize(const uint64 size)
{
    char StrSize[20];
    sprintf(StrSize,"%d",size);
    SetSize(StrSize);
}

uint32 TypeHeader::Read(IOCallback & input)
{
    try {
	binary Header[160];
	input.readFully(Header, default_size());
	uint32 result = default_size();

	if (result != default_size())
	    throw 0; // unexpected end of file

	for (int i=0; i<TYPE_HEADER_HEAD_SIZE; i ++) {
	    if (HEAD[i] != Header[i]) {
		throw 0;
	    }
	}

	char *cursor = (char*) &Header[TYPE_HEADER_HEAD_SIZE];

	myInfo = cursor;
	cursor += INFO_SIZE+TYPE_HEADER_SEP1_SIZE;
	myAds = cursor;
	cursor += ADS_SIZE+TYPE_HEADER_SEP2_SIZE;
	mySize = cursor;

	return result;
    }
    catch (exception & Ex)
    {
	throw Ex;
    }
}

END_MATROSKA_NAMESPACE
