/*
	libmatroska : parse and create Matroska files, see http://matroska.sourceforge.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
	\file
	\version \$Id: TypeHeader.hpp,v 1.7 2002/05/06 20:45:25 robux4 Exp $
	\author Ingo Ralf Blum   <ingoralfblum @ users.sf.net>
	\author Lasse Krkkinen <tronic @ users.sf.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#ifndef LIBMATROSKA_TYPE_HEADER_HPP
#define LIBMATROSKA_TYPE_HEADER_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "IOCallback.hpp"
#include "TruncString.hpp"

START_MATROSKA_NAMESPACE

const size_t TYPE_HEADER_HEAD_SIZE = 6;
const size_t TYPE_HEADER_SEP1_SIZE = 9;
const size_t TYPE_HEADER_SEP2_SIZE = 10;
const size_t TYPE_HEADER_TAIL_SIZE = 4;

/*!
    \class TypeHeader
    \brief The part of the MainHeader that can be displayed with the DOS 'type' command
    \see http://matroska.sourceforge.net/specs/#type
*/
class TypeHeader {
    public:
	TypeHeader();

	inline void SetInfo(const std::string & aStr) { myInfo = aStr; }
	inline void SetAds(const std::string & aStr) { myAds = aStr; }
	inline void SetSize(const std::string & aStr) { mySize = aStr; }
	void SetSize(const uint64 size);

	/*!
	    \return the number of bytes written to file
	*/
	uint32 Render(IOCallback & output) const;

	/*!
	    \return the number of bytes read from file
	*/
	uint32 Read(IOCallback & input);

	static inline uint32 default_size() {return 160;}

    protected:
	static const binary HEAD[TYPE_HEADER_HEAD_SIZE];
	static const size_t INFO_SIZE;
	TruncString myInfo;

	static const binary SEPARATION_1[TYPE_HEADER_SEP1_SIZE];
	static const size_t ADS_SIZE;
	TruncString myAds;

	static const binary SEPARATION_2[TYPE_HEADER_SEP2_SIZE];
	static const size_t SIZE_SIZE;
	TruncString mySize;

	static const binary TAIL[TYPE_HEADER_TAIL_SIZE];
};

END_MATROSKA_NAMESPACE

#endif // LIBMATROSKA_TYPE_HEADER_HPP
