// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.2 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "Config.hpp"
#include "StdInclude.hpp"
#include "HRESULTHolder.hpp"
#include "WindowsError.hpp"
#include "COMError.hpp"

START_MATROSKA_NAMESPACE

HRESULT HRESULTHolder::getHRESULTFromException(const std::exception&Ex)throw()
{
	const HRESULTHolder*Holder=dynamic_cast<const HRESULTHolder*>(&Ex);
	if(Holder!=0)
		return Holder->getHRESULT();

	const std::bad_alloc*Alloc=dynamic_cast<const std::bad_alloc*>(&Ex);
	if(Alloc!=0)
		return E_OUTOFMEMORY;

	return E_FAIL;
}

void HRESULTHolder::throwOnError(HRESULT Result,const char*Message)
{
	if(SUCCEEDED(Result))
		return;

	if(Result==E_OUTOFMEMORY)
		throw std::bad_alloc();

	switch(HRESULT_FACILITY(Result))
	{
	case FACILITY_WIN32:
		throw WindowsError(HRESULT_CODE(Result),Message);
	default:
		throw COMError(Result,Message);
	}
}

void HRESULTHolder::throwOnError(HRESULT Result)
{
	throwOnError(Result,"");
}

END_MATROSKA_NAMESPACE
