// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.2 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "Config.hpp"
#include "StdInclude.hpp"
#include "LocalMemory.hpp"
#include "WindowsError.hpp"

#include "../../Debug.hpp"

START_MATROSKA_NAMESPACE

LocalMemory::LocalMemory(HLOCAL Handle,bool Movable)throw()
{
	this->Handle=Handle;
	LockCount=Movable?0:-1;
}

LocalMemory::~LocalMemory()throw()
{
	try
	{
		free();
	}
	catch(std::exception&Ex)
	{
		MATROSKA_TRACE("Error: In ~LocalMemory: %hs\n",Ex.what());
	}
}

void*LocalMemory::lock()
{
	assert(Handle!=0);

	if(isMovable())
	{
		void*Address=LocalLock(Handle);
		if(Address==0)
			throw WindowsError("Can't lock local memory handle");

		++LockCount;
		return Address;
	}
	else
		return reinterpret_cast<void*>(Handle);
}

bool LocalMemory::unlock()
{
	assert(Handle!=0);

	if(isMovable())
		return false;

	assert(LockCount>0);

	bool Result=LocalUnlock(Handle)!=FALSE;
	if(!Result)
	{
		DWORD Error=GetLastError();
		if(Error!=ERROR_SUCCESS)
			throw WindowsError(Error,"Can't unlock local memory handle");
	}

	--LockCount;
	return Result;
}

void LocalMemory::free()
{
	if(Handle==0)
		return;

	if(isMovable())
	{
		while(LockCount>0)
			unlock();
	}

	if(LocalFree(Handle)!=0)
		throw WindowsError("Can't free local memory handle");

	Handle=0;
}

LocalMemory::Flags LocalMemory::getFlags()const
{
	assert(Handle!=0);

	UINT Flags=LocalFlags(Handle);
	if(Flags==LMEM_INVALID_HANDLE)
		throw WindowsError("Can't query flags for local memory handle");

	return Flags;
}

END_MATROSKA_NAMESPACE
