// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.3 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INGO_WIN32_LOCALMEMORY_HPP
#define INGO_WIN32_LOCALMEMORY_HPP

#include <windows.h>

START_MATROSKA_NAMESPACE

// Small helper class for HLOCAL handles
class LocalMemory
{
// Types
public:
	class Flags
	{
	private:
		UINT Value;

	public:
		Flags(UINT Init):Value(Init){}
		bool isDiscardable()const throw(){return (Value&LMEM_DISCARDABLE)!=0;}
		size_t getLockCount()const throw(){return Value&LMEM_LOCKCOUNT;}
	};


private:
	HLOCAL Handle;

	// The lock count of the handle. This value is
	// -1 for a fixed memory.
	size_t LockCount;

public:
	// Takeover constructor for the memory block
	LocalMemory(HLOCAL Handle=0,bool Movable=false)throw();
	~LocalMemory()throw();

	// If the memory is movable, it is locked and then a pointer to
	// the memory block is returned. Otherwise only the pointer is
	// returned.
	void*lock();

	// Unlock the handle. If it is still locked, true is returned, otherwise false
	bool unlock();

	// Free the handle. This automatically unlocks it.
	void free();

	Flags getFlags()const;

	bool isMovable()const throw(){return LockCount!=-1;}
	bool isFixed()const throw(){return LockCount==-1;}
};


END_MATROSKA_NAMESPACE

#endif
