// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.2 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "Config.hpp"
#include "StdInclude.hpp"

#include "WindowsError.hpp"
#include "LocalMemory.hpp"
#include "WindowsModule.hpp"

#include <lmerr.h>

#include <sstream>

using namespace std;

START_MATROSKA_NAMESPACE

WindowsError::WindowsError(DWORD Grund,const std::string&Description):std::runtime_error(Description)
{
	Fehler=Grund;
	WhatSet=false;
}

WindowsError::WindowsError(const std::string&Description):std::runtime_error(Description)
{
	Fehler=GetLastError();
	WhatSet=false;
}

HRESULT WindowsError::getHRESULT()const throw()
{
	return HRESULT_FROM_WIN32(Fehler);
}

const char*WindowsError::what()const throw()
{
	if(!WhatSet)
	{
		try
		{
			std::string Message=getMessage();

			if(*getDescription()==0)
				What=Message;
			else
				What=string(getDescription())+": "+Message;
		}
		catch(...)
		{
		}

		WhatSet=true;
	}

	if(What.empty())
		return getDescription();
	else
		return What.c_str();
}

string WindowsError::getMessage()const
{
	try
	{
		if(Fehler>=NERR_BASE&&Fehler<=MAX_NERR)
		{
			WindowsModule Module("netmsg.dll");
			return Module.getMessageA(Fehler);
		}
		else
		{
			HLOCAL Handle;

			if(::FormatMessageA(
				FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM
				,0
				,Fehler
				,MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT)
				,reinterpret_cast<char*>(&Handle)
				,0
				,0)==FALSE)
				throw WindowsError("Can't format message");

			LocalMemory Mem(Handle);
			//return std::string(reinterpret_cast<const char*>(Mem.lock())).trimRight(); // there is a line feed at the end of the message
			return reinterpret_cast<const char*>(Mem.lock());
		}
	}
	catch(...)
	{
		ostringstream Fmt;
		Fmt<<"Windows error 0x"<<hex<<Fehler;
		return Fmt.str();
	}
}

void SEHError::translate(unsigned int Code,EXCEPTION_POINTERS*Pointers)
{
	throw SEHError(Code);
}

END_MATROSKA_NAMESPACE


