// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.2 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INGO_WINDOWSEXCEPTION_H
#define INGO_WINDOWSEXCEPTION_H

#if _MSC_VER>=1100
#pragma once
#endif

#include "HRESULTHolder.hpp"

#include <eh.h>

#include <exception>
#include <cassert>
#include <stdexcept>


START_MATROSKA_NAMESPACE

class WindowsError:public std::runtime_error,public HRESULTHolder
{
// Variablen...
private:
	DWORD Fehler;
	mutable std::string What;
	mutable bool WhatSet;

public:
	WindowsError(DWORD Grund=GetLastError(),const std::string&Description=std::string());
	WindowsError(const std::string&Description);

	virtual const char*what()const throw();

	DWORD getError()const throw(){return Fehler;}
	const char*getDescription()const throw(){return runtime_error::what();}

	// Format the message and return it. If a message could not be found
	// a hexadecimal value is created. If that also fails, the string remains empty.
	// This method is virtual to allow derived classes to do their own message formatting.
	// Derived classes should only throw an exception, if the base implementation
	// fails, that is when something really weird happens.
	virtual std::string getMessage()const;

	// Put the error into an HRESULT value and return that.
	virtual HRESULT getHRESULT()const throw();
};

// Special class for handling SEH exceptions
class SEHError:public std::runtime_error
{
private:
	DWORD Code;

public:
	SEHError(DWORD Code)
		:std::runtime_error("SEH exception")
		,Code(Code)
	{
	}

	DWORD getCode()const throw(){return Code;}

	// This is used for the _set_se_translator function.
	static void translate(unsigned int Code,EXCEPTION_POINTERS*Pointers);
};

END_MATROSKA_NAMESPACE


#endif
