// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.3 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INGO_WINDOWSFILE_HPP
#define INGO_WINDOWSFILE_HPP

#if _MSC_VER>=1100
#pragma once
#endif

#include "../../IOCallback.hpp"
#include "../../Types.hpp"

#include "WindowsError.hpp"
#include "WindowsHandle.hpp"


START_MATROSKA_NAMESPACE

class WindowsFile:public IOCallback,public WindowsHandle
{
public:
	WindowsFile();

	// Convnience constructor
	WindowsFile(const char*FileName
		,DWORD Access
		,DWORD ShareMode
		,LPSECURITY_ATTRIBUTES SecurityAttrs
		,DWORD CreationDist
		,DWORD FlagsAndAttrs=FILE_ATTRIBUTE_NORMAL
		,HANDLE Template=0
		);

	// The parameters are the same as for CreateFile
	void open(const char*FileName
		,DWORD Access
		,DWORD ShareMode
		,LPSECURITY_ATTRIBUTES SecurityAttrs
		,DWORD CreationDist
		,DWORD FlagsAndAttrs=FILE_ATTRIBUTE_NORMAL
		,HANDLE Template=0
		);

	virtual void close();
	virtual void flush();

	virtual size_t write(const void*Mem,size_t Len);
	virtual uint32 read(void*Mem,size_t Len);

	// Combined seek method
	uint64 seek(int64 Amount,seek_mode RelTo)const ;

	// Individual seek methods
	virtual void setFilePointer(int64 Offset,seek_mode Where=seek_beginning);
	virtual uint64 getFilePointer();

	// Retrieve the file size and leave the current file pointer intact. This uses
	// the GetFileSize Windows API function.
	virtual uint64 getSize();

	// Implementation of the DeviceIOControl function. This method a synchronous call
	// to this function. The returned value is the number of bytes returned in the output
	// buffer.
	size_t ioControl(DWORD Operation,const void*InBuffer,size_t InBufferSize,void*OutBuffer,size_t OutBufferSize)const;

	// Implementation of GetFileAttributes
	static DWORD getAttributes(const char*Filename);
	static void getAttributes(const char*Path,WIN32_FILE_ATTRIBUTE_DATA&Data);

	// Shortcuts to getAttributes
	static bool isDirectory(const char*Path){return (getAttributes(Path)&FILE_ATTRIBUTE_DIRECTORY)!=0;}
	static uint64 getSize(const char*Path);

	// Implementation of GetShortPathName. According to the API documentation, the maximum path
	// length for the const char* argument is MAX_PATH. This limit can be extended by using the wchar_t
	// version by prepending a \\?\. For convenience purposes, the \\?\ is appended automatically, when
	// the path is longer than MAX_PATH in the wide version.
	static std::string getShortPath(const char*Path);

	// Implementation of the GetFullPathName function.
	static std::string getFullPath(const char*Filename);
};

void getWindowsDirectory(std::string&Result);

END_MATROSKA_NAMESPACE


#endif
