// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
// You are not allowed to remove this header from this file.
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.3 $
//
// License: LGPL
//  You should have received a copy of the GNU (Lesser) General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INGO_WIN32_WINDOWSHANDLE_HPP
#define INGO_WIN32_WINDOWSHANDLE_HPP

#include <windows.h>
#include "../../Types.hpp"

START_MATROSKA_NAMESPACE

// Mark the windows handle as noncopyable for now. Later implement a
// copy constructor.
class WindowsHandle
{
// Definitions
public:
	// Unfortunately the Windows headers define uppercase identifiers
	// so I had to use lowercase here.
	enum wait_result
	{
		wait_timed_out=0
		,wait_abandoned=1
		,wait_succeeded=2
	};

	typedef std::pair<wait_result,size_t> multi_wait_result;

// Variables...
private:
	HANDLE Handle;

// Methods...
private:
	// Implementation of the WaitForMultipleObjects call. The parameters are in
	// the same order as for WaitForMultipleObjects.
	static multi_wait_result wait(size_t Count,const HANDLE*Handles,bool WaitAll,DWORD Milliseconds);

public:
	WindowsHandle()throw();
	WindowsHandle(HANDLE Hnd)throw();
	virtual~WindowsHandle();

	void close();

	// Take over the ownership of the handle
	void attach(HANDLE Hnd);

	// Wait for the specified time in milliseconds. The method
	// then returns true, when the handle got signaled and false,
	// if the wait timed out, which means the object is not signaled.
	// A wait for an infinite amount of time always returns true or throws
	// an exception.
	wait_result wait(uint32 Milliseconds=INFINITE)const;

	// Test wether the handle is signaled with the WaitForSingleObject function.
	bool isSignaled()const;

	HANDLE getHandle()const throw();
	WindowsHandle&operator=(HANDLE Hnd);
	operator HANDLE()const throw();

	// Return true if the handle is open and false otherwise.
	bool isOpen()const throw();
	operator bool()const throw(){return isOpen();}
	bool operator!()const throw(){return !isOpen();}

	// Convenience methods for multiple waits. The return value is the number of the
	// object that was signaled, beginning with 0, or -1 to specify a timeout.
	static multi_wait_result wait(const WindowsHandle&First,const WindowsHandle&Second,DWORD Milliseconds=INFINITE,bool WaitAll=false);
};

END_MATROSKA_NAMESPACE

#endif
