// Package:
//  This file is part of the Ingo++ library. Contact Ingo Ralf Blum for more information.
//
// Copyright: Copyright Ingo Ralf Blum, 2001
//
// Authors:
//  Ingo Ralf Blum <ingoralfblum@users.sf.net>, <papierkorb@ingoralfblum.com>, <ingoralfblum@gmx.de>
// 
// File version: $Revision: 1.3 $
//
// License: LGPL
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef INGO_WIN32_WINDOWSMODULE_HPP
#define INGO_WIN32_WINDOWSMODULE_HPP

#include <string>

#include <windows.h>

START_MATROSKA_NAMESPACE

extern HMODULE Instance;

class WindowsModule
{
private:
	HMODULE Handle;
	bool Loaded;

private:
	void initialize();

public:
	WindowsModule(HMODULE Handle=libmatroska::Instance)throw();

	// Either load the specified module or simply get a handle to an
	// already loaded module.
	WindowsModule(const char*Name,bool Load=true);

	WindowsModule(const char*Name,DWORD Flags);
	~WindowsModule()throw();

	// Query an already open module. This is similar to
	// load, but it does not increment the reference count.
	// If Name is 0, the executable module is queried.
	void query(const char*Name=0);

	// Load the library with the given name
	void load(const char*Name);

	// Load the library with additional flags
	void load(const char*Name,DWORD Flags);

	// Unload the library. This is the implementation of the FreeLibrary call.
	// If the handle is 0, this does nothing. If the handle was not loaded
	// it is not freed, but only set to 0.
	void close();

	void disableThreadCalls()const ;

	// Return the specified proc address. Usually an exception is thrown whenever
	// the proc can't be located, so the returned value is always != 0, unless the
	// ReturnNullIfNotFound flag is set to true. In that case 0 is returne, if the specified
	// module does not contain the specified proc. However, all other errors still yield
	// an exception.
	FARPROC getProcAddress(const char*Name,bool ReturnNullIfNotFound=false)const ;

	bool isLoaded()const throw(){return Handle!=0;}
	operator bool()const throw(){return isLoaded();}
	bool operator!()const throw(){return !isLoaded();}

	// Resource functions. These also work with an empty module handle, which
	// then specifies the module of the exe, that created the process.

	// Returns the resource handle for the specified resource or throws
	// anerror. The handle can be used until the module is unloaded.
	// If the module is 0, the resource is valid until the process exits.
	HRSRC findResource(const char*Type,int ID)const ;
	void*getResourcePointer(HRSRC Resource)const ;
	size_t getResourceSize(HRSRC Resource)const ;

	std::string getMessageA(DWORD MessageID,DWORD Language=0);

	std::string getFilenameA();
};

END_MATROSKA_NAMESPACE


#endif
