/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: test1.cpp,v 1.10 2002/05/09 13:38:05 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/

//#include <cstdio>

#include <iostream>

#include "BlockFxS.hpp"
#include "BlockOcL.hpp"
#include "BlockSPB.hpp"
#include "StdIOCallback.hpp"

using namespace LIBMATROSKA_NAMESPACE;
using namespace std;

/*void test(Block::add_result result)
{
    switch(result)
    {
    case Block::BLOCK_ADD_COMPLETE:       ///< This block is full now
	printf("Data added, no space for more\n");
	break;
    case Block::BLOCK_ADD_DONE:           ///< The frame has been added, more frames can be added
	printf("Data added\n");
	break;
    case Block::BLOCK_ADD_ERR_BLOCK_FULL: ///< This block was already full
	printf("Failed : block is full of data\n");
	break;
    case Block::BLOCK_ADD_ERR_USE_OTHER:  ///< Use another block to store the frame
	printf("Failed : the parameters of the block are not the same as the data\n");
	break;
    case Block::BLOCK_ADD_ERR_MEMORY_FULL: ///< No more memory on device
	printf("Failed : no more memory left\n");
	break;
    case Block::BLOCK_ADD_ERR_BAD_FRAME_LEN: ///< No more memory on device
	printf("Failed : frame size not corresponding\n");
	break;
    }
}*/

/*!
    \test Test the filling of a Block with frames/chuncks
*/
int main(void)
{
    /* create some test buffers */
    const char string1[] = "The first data for the frame";

    const char string2[] = "The second data for the frame which is much bigger";

    binary * data1=new binary[2000];
    memset(data1,0xA5,2000);

    StdIOCallback OCL_file("block-ocl.mcf",StdIOCallback::MODE_CREATE);
    StdIOCallback SPB_file("block-spb.mcf",StdIOCallback::MODE_CREATE);
    StdIOCallback FXS_file("block-fxs.mcf",StdIOCallback::MODE_CREATE);

    /* create the block */
    uint8 TrackNumber = 1;
    uint32 Timecode = 0;
    bool bKeyframe = 1;
    bool bBidirFrame = 0;

    Adler32 globalChecksum;

    bool add_res;

    try {
	BlockOcL a_blockOCL(TrackNumber,Timecode);

	try {
	   printf("***Testing Block Octet Lacing***\n");
	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockOCL.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",255);
	    add_res = a_blockOCL.AddFrame(data1, 255, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockOCL.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockOCL.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	a_blockOCL.Render(OCL_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }


    try {
	BlockSPB a_blockSPB(TrackNumber,Timecode);

	try {

	   printf("\n***Testing Block Simple Lacing***\n");
	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockSPB.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockSPB.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockSPB.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	a_blockSPB.Render(SPB_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }


    try {
        BlockFxS a_blockFXS(TrackNumber,Timecode,2000);

	try {

	   printf("\n***Testing Block Fixed Size Lacing***\n");
	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockFXS.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockFXS.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockFXS.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	// we can delete it now since the data are already buffered in memory
	delete data1;

	a_blockFXS.Render(FXS_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }

    return 0;
}
