/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: test2.cpp,v 1.8 2002/05/14 11:43:37 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include "Cluster.hpp"
#include "StdIOCallback.hpp"
#include "Track.hpp"
#include "TrackAudio.hpp"

#include <memory.h>
#include <algorithm>

#ifdef _MSC_VER
#include <windows.h>      // for min/max
#endif // _MSC_VER

using namespace LIBMATROSKA_NAMESPACE;
using namespace std;

/*void test(Cluster::add_result result)
{
    switch(result)
    {
    case Cluster::CLUSTER_ADD_DONE:           ///< The frame has been added, more frames can be added
	printf("Data added\n");
	break;
    case Cluster::CLUSTER_ADD_ERR_BLOCK_FULL: ///< the Cluster is full, it should be rendered
	printf("Failed : cluster is full of data\n");
	break;
    case Cluster::CLUSTER_ADD_ERR_MEMORY_FULL:///< no more memory on device
	printf("Failed : no more memory\n");
	break;
    }
}*/

/*!
    \test Some test on the Cluster use
    \todo render the Cluster to a file
*/
int main(void)
{
    binary *audio,*video;
    audio = new binary[5000];
    video = new binary[10000];

    memset(audio, 'a', 5000);
    memset(video, 'V', 10000);

    const uint16 audio_chunk_size = 2000, audio_total = 5000;
    uint16 audio_remain = audio_total;
    const uint16 video_chunk_size = 2000, video_total = 10000;
    uint16 video_remain = video_total;

    Cluster a_Cluster(10000);
//    Cluster::add_result add_res;
    bool add_res;
    TrackAudio trackType;
    Track TestTrack(trackType);

    StdIOCallback Cluster_file("cluster.mcf",StdIOCallback::MODE_CREATE);

    // attempt to mux audio and video
    while (1)
    {
	if (video_remain != 0) // add a chunck of video to the cluster
	{
	    uint16 size_to_add = min(video_remain,video_chunk_size);
	    printf("adding video chunck of %04d bytes to the cluster\n",size_to_add);
	    add_res = TestTrack.AddFrame((video_total-video_remain)/10, // timecode
			       &video[video_total-video_remain], // the buffer start
			       size_to_add, // size of the buffer
			       true, // keyframe
			       false); // B frame
//	    test(add_res);
	    video_remain -= size_to_add;
	}

	if (audio_remain != 0) // add a chunck of audio to the cluster
	{
	    uint16 size_to_add = min(audio_remain,audio_chunk_size);
	    printf("adding audio chunck of %04d bytes to the cluster\n",size_to_add);
	    add_res = TestTrack.AddFrame((audio_total-audio_remain)/10, // timecode
			       &audio[audio_total-audio_remain], // the buffer start
			       size_to_add, // size of the buffer
			       true, // keyframe
			       false); // B frame
//	    test(add_res);
	    audio_remain -= size_to_add;
	}

	if (video_remain == 0 && audio_remain == 0)
	    break;
    }

    delete audio;
    delete video;

    uint32 NumberOfBlock;
    a_Cluster.Render(Cluster_file, NumberOfBlock);
    printf("%d blocks written\n",NumberOfBlock);

    return 0;
}
