#include <cstdio>

#include "Types.hpp"

using namespace LIBMATROSKA_NAMESPACE;

typedef struct mix_test {
    mix_test()
     :lu32(uint32(0)),bu16(uint16(0)){}

    lil_uint32 lu32;
    big_uint16 bu16;
} mix_test;

int main(void)
{
    // part 1 test your machine
    uint32 endian_test;
    char* fill_pt = (char*)&endian_test;
    
    fill_pt[0] = 0x01;
    fill_pt[1] = 0x02;
    fill_pt[2] = 0x03;
    fill_pt[3] = 0x04;

    if (endian_test == 0x01020304)
	printf("This machine is big-endian (PPC, Alpha, 68xxx)\n");
    else
	printf("This machine is little-endian (Intel x86)\n");

    printf("\n");

    // part 2 test the classes
    lil_int16  lil_16  = 0x0102;
    lil_uint16 lil_u16 = 0x0304;
    lil_int32  lil_32  = 0x05060708;
    lil_uint32 lil_u32 = 0x090A0B0C;
    big_int16  big_16  = 0x0102;
    big_uint16 big_u16 = 0x0304;
    big_int32  big_32  = 0x05060708;
    big_uint32 big_u32 = 0x090A0B0C;

    int16  test_lil16 = lil_16;
    int16  endn_lil16 = lil_16.endian();
    int16 *buff_lil16 = &lil_16.endian();
    size_t size_lil16 = lil_16.size();

    printf("Little-endian test\n");
    printf("0x%04X native = 0x%04X little-endian buffer, size for int16: %d\n",test_lil16,*buff_lil16,size_lil16);

    int16  test_big16 = big_16;
    int16  endn_big16 = big_16.endian();
    int16 *buff_big16 = &big_16.endian();
    size_t size_big16 = big_16.size();

    printf("Big-endian test\n");
    printf("0x%04X native = 0x%04X big-endian buffer, size for int16: %d\n",test_big16,*buff_big16,size_big16);
    
    printf("\n");

    // part 3 structure test
    mix_test item;
    item.lu32 = 0xF0F1F2F3;
    item.bu16 = 0xF4F5;

    printf("Structure little-endian uint32 / big-endian uint16\n");
    printf("Platform values in the structure : 0x%08X / 0x%04X\n", uint32(item.lu32), uint16(item.bu16));
    printf("Buffered values in the structure : 0x%08X / 0x%04X\n", item.lu32.endian(), item.bu16.endian());

    // part 4 process from a buffer

    printf("\n");

    uint32 lil_buffer_endn;
    binary *lil_buf_endn = (binary*)&lil_buffer_endn;

    uint32 big_buffer_endn;
    binary *big_buf_endn = (binary*)&big_buffer_endn; 

    lil_buf_endn[0] = big_buf_endn[0] = 0x55;
    lil_buf_endn[1] = big_buf_endn[1] = 0x44;
    lil_buf_endn[2] = big_buf_endn[2] = 0x22;
    lil_buf_endn[3] = big_buf_endn[3] = 0x00;

    lil_uint32 l32_from_buf;
    big_uint32 b32_from_buf;

    l32_from_buf.Eval(lil_buf_endn);
    b32_from_buf.Eval(big_buf_endn);

    printf("The little-endian should be reversed, the big endian should be the same\n");
    printf("Platform values : little 0x%08X / big 0x%08X\n", uint32(l32_from_buf), uint32(b32_from_buf));
    printf("Buffered values : little 0x%02X%02X%02X%02X / big 0x%02X%02X%02X%02X\n",
							    ((uint8*)(&l32_from_buf.endian()))[0],
							    ((uint8*)(&l32_from_buf.endian()))[1],
							    ((uint8*)(&l32_from_buf.endian()))[2],
							    ((uint8*)(&l32_from_buf.endian()))[3],
							    ((uint8*)(&b32_from_buf.endian()))[0],
							    ((uint8*)(&b32_from_buf.endian()))[1],
							    ((uint8*)(&b32_from_buf.endian()))[2],
							    ((uint8*)(&b32_from_buf.endian()))[3]);

    return 0;
}
