/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: test6.c,v 1.2 2002/05/17 10:41:43 robux4 Exp $
    \brief Test muxing two tracks into valid clusters/blocks/frames
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/

#ifdef _MSC_VER
#include <windows.h>      // for min/max
#endif // _MSC_VER

#include <stdio.h>

#include "libmatroska.h"

#define BIN_FILE_SIZE  15000
#define TXT_FILE_SIZE   3000
#define BIN_FRAME_SIZE  1500
#define TXT_FRAME_SIZE   200

/*!
    The first file is a "binary" file with data scaling from 0x00 to 0xFF repeatedly
    The second file is a "text" file with data scaling from 'z' to 'a'
*/
int main(int argc, char **argv)
{
    char *file1;
    char *file2;
    FILE *fileid;
    matroska_id EncodedFile;

    if (argc == 1)
    {
	char c;
	unsigned int i;

	file1 = "binr.bin";
	file2 = "text.bin";

	// create the files
	fileid = fopen(file1,"wb+");
	for (i=0; i<BIN_FILE_SIZE; i++)
	{
	    c = i % 0x100;
	    fputc(c, fileid);
	}
	fclose(fileid);

	fileid = fopen(file2,"wb+");
	for (i=0; i<TXT_FILE_SIZE; i++)
	{
	    c = (i % 26) + 'a';
	    fputc(c, fileid);
	}
	fclose(fileid);
    }
    else if(argc < 3)
    {
	printf("Syntax: mcfmux [binfile txtfile]\n");
	return 1; // error
    }
    else
    {
	file1 = argv[1];
	file2 = argv[2];
    }

    printf("Creating \"muxed.mcf\"\n");

    // create the Matroska file
    EncodedFile = matroska_open_file("muxed.mcf", "wb+");

#ifdef OLD
    // add the binary track (audio)
    TrackAudio AudioTrack;
    TrackID binID = MuxedFile.CreateTrack( AudioTrack );
    MuxedFile.track_SetName(binID, "Audio test");
    MuxedFile.track_SetLaced(binID);

    TrackInfoAudio aAudioTrack;
    memset(&aAudioTrack, 0, sizeof(aAudioTrack));
    aAudioTrack.SampleRate = 44100;
    aAudioTrack.BitDepth = 8;
    aAudioTrack.ChannelPerSubtrack = 1;
    aAudioTrack.DTS = false;
    aAudioTrack.SecondaryTrack = false;
    aAudioTrack.SubtrackNumber = 1;
    aAudioTrack.ChannelPosition = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;

    MuxedFile.Track_SetInfo_Audio(binID, aAudioTrack);

    // add the text track (subtitle)
    TrackSubtitle SubtitleTrack;
    TrackID txtID = MuxedFile.CreateTrack( SubtitleTrack );
    MuxedFile.track_SetName(txtID, "Subtitle test");

    MuxedFile.RenderHead("libmatroska test6.cpp");

    // to verify the muxing is working
    MuxedFile.SetMaxClusterSize(2000);

    // start muxing (2 binary frames for each text frame)
    StdIOCallback bin_file(file1,StdIOCallback::MODE_READ);
    StdIOCallback txt_file(file2,StdIOCallback::MODE_READ);
    
    bin_file.setFilePointer(0L, seek_end);
    txt_file.setFilePointer(0L, seek_end);
    
    BIN_FILE_SIZE = bin_file.getFilePointer();
    TXT_FILE_SIZE = txt_file.getFilePointer();
    
    binary *buf_bin = new binary[BIN_FILE_SIZE];
    binary *buf_txt = new binary[TXT_FILE_SIZE];

    bin_file.setFilePointer(0L, seek_beginning);
    txt_file.setFilePointer(0L, seek_beginning);

    bin_file.read(buf_bin, BIN_FILE_SIZE);
    txt_file.read(buf_txt, TXT_FILE_SIZE);

    binary *curs_bin = buf_bin;
    binary *curs_txt = buf_txt;
    unsigned int remain_bin = BIN_FILE_SIZE;
    unsigned int remain_txt = TXT_FILE_SIZE;

    while (remain_bin !=0 || remain_txt != 0)
    {
	// a text chunck
	if (remain_txt !=0) {
	    MuxedFile.AddFrame(txtID, 0, curs_txt, min(TXT_FRAME_SIZE,remain_txt));
	    curs_txt += min(TXT_FRAME_SIZE,remain_txt);
	    remain_txt -= min(TXT_FRAME_SIZE,remain_txt);
	}
	// a binary chunck
	if (remain_bin !=0) {
	    MuxedFile.AddFrame(binID, 0, curs_bin, min(BIN_FRAME_SIZE,remain_bin));
	    curs_bin += min(BIN_FRAME_SIZE,remain_bin);
	    remain_bin -= min(BIN_FRAME_SIZE,remain_bin);
	}
	// a text chunck
	if (remain_txt !=0) {
	    MuxedFile.AddFrame(txtID, 0, curs_txt, min(TXT_FRAME_SIZE,remain_txt));
	    curs_txt += min(TXT_FRAME_SIZE,remain_txt);
	    remain_txt -= min(TXT_FRAME_SIZE,remain_txt);
	}
    }

    delete[] buf_bin;
    delete[] buf_txt;
#endif /* OLD */

    matroska_end(EncodedFile, 1000); // 1000 ms

    return 0;
}
