/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: test7.cpp,v 1.3 2002/07/14 20:47:46 robux4 Exp $
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include <iostream>

//#include "Cluster.hpp"
#include "FileKax.hpp"
/*#include "BlockFxS.hpp"
#include "BlockOcL.hpp"
#include "BlockSPB.hpp"*/
#include "StdIOCallback.hpp"

using namespace LIBMATROSKA_NAMESPACE;
using namespace std;

/*!
    \test Test reading frames/chuncks from a File
    \note you can generate the file used in this example using test6.cpp
*/
int main(void)
{
    try {
	StdIOCallback Cluster_file("muxed.kav", ::MODE_READ);
    
	// create the Matroska file
	FileKax MuxedFile(Cluster_file);

	MuxedFile.ReadHead();
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }

#if 0
	Cluster a_Cluster(10000);

	uint32 offset = a_Cluster.FindHead(Cluster_file);
	if (offset != 0)
	    cout << "There was of trash bytes before the cluster was found" << endl;

	const unsigned int MAX_FRAME_SIZE = 5000;
	binary data1[MAX_FRAME_SIZE];*
	
	uint32 size_read = a_Cluster.Read(Cluster_file);

    StdIOCallback OCL_file("block-ocl.mcf","r");
    StdIOCallback SPB_file("block-spb.mcf","r");
    StdIOCallback FXS_file("block-fxs.mcf","r");

    uint8 TrackNumber = 1;

    try {
	BlockSPB a_blockSPB;
	size_read = MAX_FRAME_SIZE;
	a_blockSPB.ReadHeader(SPB_file);
//	a_blockSPB.ReadFrame(data1, size_read);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }

    /* create some test buffers */
    const char string1[] = "The first data for the frame";

    const char string2[] = "The second data for the frame which is much bigger";

    binary * data1=new binary[2000];
    memset(data1,0xA5,2000);

    /* create the block */
    uint8 TrackNumber = 1;
    uint32 Timecode = 0;
    bool bKeyframe = 1;
    bool bBidirFrame = 0;

    Adler32 globalChecksum;

    bool add_res;

    try {
	BlockOcL a_blockOCL(TrackNumber,Timecode);

	try {
	   printf("***Testing Block Octet Lacing***\n");
	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockOCL.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",255);
	    add_res = a_blockOCL.AddFrame(data1, 255, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockOCL.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockOCL.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	a_blockOCL.Render(OCL_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }


    try {
	BlockSPB a_blockSPB(TrackNumber,Timecode);

	try {

	   printf("\n***Testing Block Simple Lacing***\n");
	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockSPB.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockSPB.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockSPB.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	a_blockSPB.Render(SPB_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }


    try {
        BlockFxS a_blockFXS(TrackNumber,Timecode,2000);

	try {

	   printf("\n***Testing Block Fixed Size Lacing***\n");
	    printf("Add block of size %04d\n",2000);
	    add_res = a_blockFXS.AddFrame(data1, 2000, bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string1));
	    add_res = a_blockFXS.AddFrame((const binary *)(string1), countof(string1), bKeyframe, bBidirFrame);

	    printf("Add block of size %04d\n",countof(string2));
	    add_res = a_blockFXS.AddFrame((const binary *)(string2), countof(string2), bKeyframe, bBidirFrame);
	}
	catch (exception & Ex)
	{
	    cout << Ex.what() << endl;
	}

	// we can delete it now since the data are already buffered in memory
	delete data1;

	a_blockFXS.Render(FXS_file, globalChecksum);
    }
    catch (exception & Ex)
    {
	cout << Ex.what() << endl;
    }
#endif
    return 0;
}
