/*
	libmatroska : parse and create Matroska files, see http://matroska.sf.net/
	Copyright (C) 2002 Steve Lhomme

	This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*!
    \file
    \version \$Id: test8.c,v 1.3 2002/05/17 18:02:18 robux4 Exp $
    \brief Test reading of the Header
    \author Lasse Krkkinen <tronic @ users.sf.net>
    \author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include <malloc.h>

#include "libmatroska.h"

#ifdef OLD
void displayTrackInfo(const TrackInfo & aTrackInfo)
{
    cout << "Track name  : " << aTrackInfo.Name << endl;
    cout << "enabled     : " << (aTrackInfo.Enabled?"yes":"no") << endl;
    cout << "preferred   : " << (aTrackInfo.Preferred?"yes":"no") << endl << endl;
}

void displayAudioInfo(const TrackInfoAudio & aAudioTrack)
{
    cout << "sample rate : " << (int)aAudioTrack.SampleRate << endl;
    cout << "bit depth   : " << (int)aAudioTrack.BitDepth << endl;
    cout << "channel/tra : " << (int)aAudioTrack.ChannelPerSubtrack << endl;
    cout << "channel pos : " << (int)aAudioTrack.ChannelPosition << endl;
    cout << "DTS         : " << (aAudioTrack.DTS?"yes":"no") << endl;
    cout << "secondary   : " << (aAudioTrack.SecondaryTrack?"yes":"no") << endl;
    cout << "subtrack #  : " << (int)aAudioTrack.SubtrackNumber << endl;
    cout << "AVI format  : " << (int)aAudioTrack.AVIFormatTag << endl << endl;
}
#endif /* OLD */

/*!
    \note you can generate the file used in this example using test6.cpp
*/
int main(void)
{
    matroska_stream File;
    matroska_id EncodedFile;
    uint8 NumberOfTracks,TrackIndex;
    matroska_track *track_table;
    track_info track_info_audio;

    File = matroska_open_stream_file("muxed.mcf", MODE_READ);
    EncodedFile = matroska_open_stream(File);

    matroska_read_head( EncodedFile );
    matroska_read_tracks( EncodedFile );

    NumberOfTracks = matroska_get_number_track( EncodedFile );

    /* get all the track access */
    track_table = malloc(NumberOfTracks * sizeof(matroska_track));

    for (TrackIndex=0; TrackIndex<NumberOfTracks; TrackIndex++)
    {
	track_table[TrackIndex] = matroska_get_track(EncodedFile, TrackIndex);
    }

    /* find the audio track */
    for (TrackIndex=0; TrackIndex<NumberOfTracks; TrackIndex++)
    {
	matroska_get_track_info( EncodedFile, track_table[TrackIndex], &track_info_audio );
	if (track_info_audio.TrackType == track_audio)
	    break;
    }

#ifdef OLD
    TrackInfo *Tracks = new TrackInfo[TrackNumber];
    TrackInfoAudio aAudioTrack;

    displayTrackInfo(Tracks[0]);

    if (Tracks[0].TrackType == track_audio) {
	MuxedFile.Track_GetInfo_Audio(track1, aAudioTrack);
	displayAudioInfo(aAudioTrack);
    }
    else
	cout << "the expected audio track is not an audio one :(" << endl;

    // let's read only track1 (audio)
    MuxedFile.SelectReadingTrack(track1);
    MuxedFile.SelectReadingTrack(track2);
    MuxedFile.SelectReadingTrack(track1,false);
    MuxedFile.SelectReadingTrack(track1);
    MuxedFile.SelectReadingTrack(track2,false);
    MuxedFile.SelectReadingTrack(track2);

    // read the frames from the selected tracks and put them in an output file
    StdIOCallback Output_file1("out-binr.bin",StdIOCallback::MODE_CREATE);
    StdIOCallback Output_file2("out-text.bin",StdIOCallback::MODE_CREATE);

    TrackID TrackRead;
    uint32 timecode; // not used yet
    binary *aFrame;
    uint32 aFrameSize;
    while (MuxedFile.ReadFrame(TrackRead, timecode, aFrame, aFrameSize))
    {
	if (TrackRead == track1)
	{
	    Output_file1.write(aFrame, aFrameSize);
	}
	else if (TrackRead == track2)
	{
	    Output_file2.write(aFrame, aFrameSize);
	}
	else cout << "received a frame from an unwanted track" << endl;
    }
#endif /* OLD */
    matroska_close(EncodedFile); // 1000 ms

    return 0;
}
