/**
    \file fio.cpp

    A class for I/O into a file
*/

#include "fio.hpp"

#include <cassert>
#include <stdio.h>

using namespace LIBMATROSKA_NAMESPACE;

IO_file::IO_file()
 :h_file(NULL)
 ,m_filename("")
 ,position(0)
{}

IO_file::~IO_file()
{
    Close();
}

int IO_file::Open(std::string & filename, int mode)
{
    enum open_mode format;
    if ((mode & FILE_MODE_READ) == FILE_MODE_READ)
    {
	    format = MODE_READ;
    }

    if ((mode & FILE_MODE_WRITE) == FILE_MODE_WRITE)
    {
        if ((mode & FILE_MODE_EXIST) == FILE_MODE_EXIST)
	{
	    format = MODE_WRITE;
	}
	else
	{
	    format = MODE_CREATE;
	}
    }

    h_file = new StdIOCallback(filename.c_str(), format);
    
    m_filename = filename;
    
    return int(h_file);
}
	
int IO_file::Close()
{
    if (h_file != NULL)
    {
	h_file->close();
	h_file = NULL;
    }

    return 0;
}

int IO_file::Read(void * buffer, size_t buffer_size)
{
    assert(h_file != NULL);

    size_t result = h_file->read(buffer, buffer_size);

    position += buffer_size;

    if (result != buffer_size)
    {
	printf("Unexpected End Of File in %s at position %ld\n",m_filename.c_str(),h_file->getFilePointer());
	return 0;
    }

    return result;
}

uint32 IO_file::ReadUpTo(void * buffer, size_t buffer_size)
{
    assert(h_file != NULL);

    size_t result = h_file->read(buffer, buffer_size);

    position += buffer_size;

    return result;
}

bool IO_file::SeekUp(uint64 SeekSize)
{
    assert(h_file != NULL);

    h_file->setFilePointer(SeekSize, seek_current);

    return true;
}
