/**
    \file
    Classes corresponding to RIFF handling
*/

#include "mcfwav.hpp"

#include "Types.hpp"

using namespace LIBMATROSKA_NAMESPACE;

const FourCC FMT_chunk::fmtFourCC('f','m','t',' ');
/*
FMT_chunk::FMT_chunk()
 :RIFF_chunk(fmtFourCC)
 ,myChannels(0)
 ,mySamplerate(0)
 ,myBytetrate(0)
 ,myBlockAlign(0)
 ,myBitPerSample(0)
 ,ExtraCodecData(NULL)
 ,myExtraCodecDataSize(0)
{
}
*/
FMT_chunk::FMT_chunk(const RIFF_chunk *aChunk)
 :RIFF_chunk(*aChunk)
 ,myChannels(0)
 ,mySamplerate(0)
 ,myBytetrate(0)
 ,myBlockAlign(0)
 ,myBitPerSample(0)
 ,ExtraCodecData(NULL)
 ,myExtraCodecDataSize(0)
{
}

FMT_chunk::~FMT_chunk()
{
    if (ExtraCodecData != NULL)
    {
	delete ExtraCodecData;
    }
}

bool FMT_chunk::Read(IO_file & file)
{
    if (myFollowingSize < 16)
    {
	printf("Don't know how to handle the format data\n");
	return false;
    }

    binary *totalBuffer = new binary[16];
    if (totalBuffer == NULL)
    {
	printf("No more space to read Format header\n");
	return false;
    }

    if (!file.Read(totalBuffer, 16))
    {
	printf("Impossible to read Format header\n");
	return false;
    }

    binary *seekPt = totalBuffer;
    myFormat[0] = *seekPt++;
    myFormat[1] = *seekPt++;

    lil_uint16 lu16;
    lu16.Eval(seekPt);
    myChannels = lu16;
    seekPt += sizeof(uint16);

    lil_uint32 lu32;
    lu32.Eval(seekPt);
    mySamplerate = lu32;
    seekPt += sizeof(uint32);

    lu32.Eval(seekPt);
    myBytetrate = lu32;
    seekPt += sizeof(uint32);

    lu16.Eval(seekPt);
    myBlockAlign = lu16;
    seekPt += sizeof(uint16);

    lu16.Eval(seekPt);
    myBitPerSample = lu16;
    seekPt += sizeof(uint16);

    myExtraCodecDataSize = myFollowingSize - 16;
    if (myExtraCodecDataSize == 0)
    {
	ExtraCodecData = NULL;
    }
    else
    {
	ExtraCodecData = new binary[myExtraCodecDataSize];
	if (ExtraCodecData == NULL)
	{
	    printf("No more space to read Format header\n");
	    return false;
	}

	if (!file.Read(ExtraCodecData, myExtraCodecDataSize))
	{
	    printf("Impossible to read Codec Data\n");
	    return false;
	}
    }

    delete totalBuffer;
    
    return true;
}